package org.obm.sync.services;

import java.util.Date;
import java.util.List;

import org.obm.sync.auth.AccessToken;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.base.Category;
import org.obm.sync.base.KeyList;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.items.EventChanges;

public interface ICalendar {

	public CalendarInfo[] listCalendars(AccessToken token) throws ServerFault,
			AuthFault;

	public Event removeEvent(AccessToken token, String calendar, String uid)
			throws AuthFault, ServerFault;

	public Event modifyEvent(AccessToken token, String calendar, Event event,
			boolean updateAttendees) throws AuthFault, ServerFault;

	public String createEvent(AccessToken token, String calendar, Event event)
			throws AuthFault, ServerFault;

	public EventChanges getSync(AccessToken token, String calendar,
			Date lastSync) throws AuthFault, ServerFault;

	public Event getEventFromId(AccessToken token, String calendar, String id)
			throws AuthFault, ServerFault;

	public KeyList getEventTwinKeys(AccessToken token, String calendar,
			Event event) throws AuthFault, ServerFault;

	public String getUserEmail(AccessToken token) throws AuthFault, ServerFault;

	public KeyList getRefusedKeys(AccessToken token, String calendar, Date since)
			throws AuthFault, ServerFault;

	public List<Category> listCategories(AccessToken at) throws AuthFault,
			ServerFault;

	public Event getEventFromExtId(AccessToken token, String calendar,
			String extId) throws AuthFault, ServerFault;

	public List<Event> getListEventsFromIntervalDate(AccessToken token,
			String calendar, Date start, Date end) throws AuthFault,
			ServerFault;

	public List<Event> getAllEvents(AccessToken token, String calendar, EventType eventType)
			throws AuthFault, ServerFault;
	
	public List<EventTimeUpdate> getAllEventTimeUpdate(AccessToken token, String calendar, EventType eventType)
		throws AuthFault, ServerFault;

	
	
	
	public String parseEvent(AccessToken token, Event event) throws ServerFault, AuthFault;
	
	public String parseEvents(AccessToken token, List<Event> events) throws ServerFault, AuthFault;
	
	public List<Event> parseICS(AccessToken token, String ics) throws Exception, ServerFault;
}
