/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.calendar;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TimeZone;

import javax.transaction.UserTransaction;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.book.Contact;
import org.obm.sync.book.Email;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventOpacity;
import org.obm.sync.calendar.EventRecurrence;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.calendar.ParticipationRole;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.calendar.RecurrenceKind;
import org.obm.sync.items.EventChanges;

import fr.aliacom.obm.common.FindException;
import fr.aliacom.obm.common.ObmDomain;
import fr.aliacom.obm.common.ObmUser;
import fr.aliacom.obm.common.UserManagement;
import fr.aliacom.obm.common.contact.ContactHome;
import fr.aliacom.obm.utils.Helper;
import fr.aliacom.obm.utils.Ical4jHelper;
import fr.aliacom.obm.utils.LinkedEntity;
import fr.aliacom.obm.utils.ObmHelper;

/**
 * Calendar data access functions
 * 
 */
public class CalendarHome {
	private static CalendarHome instance = new CalendarHome();

	/**
	 * @return the home instance
	 */
	public static CalendarHome getInstance() {
		return instance;
	}

	// event_ext_id | character varying(255) | default ''::character varying
	// event_owner | integer |
	// event_timezone | character varying(255) | default 'GMT'::character
	// varying
	// event_opacity | event_opacity | default 'OPAQUE'::event_opacity
	// event_title | character varying(255) | default NULL::character varying
	// event_location | character varying(100) | default NULL::character varying
	// event_category1_id | integer |
	// event_priority | integer |
	// event_privacy | integer |
	// event_date | timestamp without time zone | not null
	// event_duration | integer | not null default 0
	// event_allday | boolean | default false
	// event_repeatkind | character varying(20) | default NULL::character
	// varying
	// event_repeatfrequence | integer |
	// event_repeatdays | character varying(7) | default NULL::character varying
	// event_endrepeat | timestamp without time zone |
	// event_color | character varying(7) |
	// event_completed | timestamp without time zone | not null
	// event_url | text |
	// event_description | text |
	// event_properties | text |

	private Log logger = LogFactory.getLog(getClass());

	private static final String EVENT_SELECT_FIELDS = "event_id, "
			+ "event_timeupdate, "
			+ "event_type, "
			+ "event_parent_id, "
			+ "event_ext_id, "
			+ "event_timezone, "
			+ "event_opacity, "
			+ "event_title, "
			+ "event_location, "
			+ "eventcategory1_label, "
			+ "event_priority, "
			+ "event_privacy, "
			+ "event_date, "
			+ "event_duration, "
			+ "event_allday, "
			+ "event_repeatkind, "
			+ "event_repeatfrequence, "
			+ "event_repeatdays, "
			+ "event_endrepeat, "
			+ "event_color, "
			+ "event_completed, "
			+ "event_url, "
			+ "event_description, now() as last_sync, event_domain_id, evententity_entity_id, "
			+ "userobm_login, " + "domain_name";

	private static final String EVENT_INSERT_FIELDS = "event_owner, "
			+ "event_parent_id, "
			+ "event_ext_id, "
			+ "event_timezone, "
			+ "event_opacity, "
			+ "event_title, "
			+ "event_location, "
			+ "event_category1_id, "
			+ "event_priority, "
			+ "event_privacy, "
			+ "event_date, "
			+ "event_duration, "
			+ "event_allday, "
			+ "event_repeatkind, "
			+ "event_repeatfrequence, "
			+ "event_repeatdays, "
			+ "event_endrepeat, "
			+ "event_color, "
			+ "event_completed, "
			+ "event_url, "
			+ "event_description, event_domain_id, event_usercreate, event_origin, event_type, event_timecreate";

	private static final String ATT_AND_ALERT_FIELDS = "eventlink_event_id, "
			+ "eventlink_state, "
			+ "eventlink_required, "
			+ "eventlink_percent, "
			+ "eventalert_duration, userobm_email, userobm_firstname, userobm_lastname, userentity_user_id ";

	private static final String CONTACT_AND_ALERT_FIELDS = "eventlink_event_id, "
			+ "eventlink_state, "
			+ "eventlink_required, "
			+ "eventlink_percent, "
			+ "0 as eventalert_duration, MAX(email_address) as userobm_email, contact_firstname as userobm_firstname, "
			+ "contact_lastname as userobm_lastname, contactentity_contact_id as userentity_user_id ";

	private static final String ATT_INSERT_FIELDS = "eventlink_event_id, eventlink_entity_id, "
			+ "eventlink_state, eventlink_required, eventlink_percent, eventlink_usercreate";

	private static final String EXCEPS_FIELDS = "event_id, eventexception_date";

	private CalendarHome() {
	}

	private String buildEventId(List<Event> changedEvent) {
		StringBuilder sb = new StringBuilder(10 * changedEvent.size());
		sb.append("0");
		for (Event e : changedEvent) {
			sb.append(",");
			sb.append(e.getDatabaseId());
		}
		return sb.toString();
	}

	private Integer catIdFromString(Connection con, String category,
			int domainId) throws SQLException {
		if (category == null) {
			return null;
		}

		Integer ret = null;
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = con
					.prepareStatement("select eventcategory1_id from EventCategory1 where eventcategory1_label=? "
							+ "and eventcategory1_domain_id=?");
			st.setString(1, category);
			st.setInt(2, domainId);
			rs = st.executeQuery();
			if (rs.next()) {
				ret = rs.getInt(1);
			}
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
		return ret;
	}

	/**
	 * @param ev
	 * @return
	 */
	public Event createEvent(AccessToken editor, Event ev) {
		logger.info("create with token " + editor.getSessionId() + " from "
				+ editor.getOrigin() + " for " + editor.getEmail());

		Connection con = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			createEvent(con, editor, ev);

			ut.commit();
		} catch (Throwable e) {
			ObmHelper.rollback(ut);
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, null, null);
		}
		return ev;
	}

	public Event createEvent(Connection con, AccessToken editor, Event ev)
			throws SQLException {
		String evQ = "INSERT INTO Event (" + EVENT_INSERT_FIELDS
				+ ") values ( " + editor.getObmId() + ",  " +
				// event_owner
				"?, " + // event_parent_id
				"?, " + // event_ext_id
				"?, " + // event_timezone
				"?, " + // event_opacity
				"?, " + // event_title
				"?, " + // event_location
				"?, " + // event_category1_id
				"?, " + // event_priority
				"?, " + // event_privacy
				"?, " + // event_date
				"?, " + // event_duration
				"?, " + // event_allday
				"?, " + // event_repeatking
				"?, " + // event_repeatfrequence
				"?, " + // event_repeatdays
				"?, " + // event_endrepeat
				"?, " + // event_color
				"?, " + // event_completed
				"?, " + // event_url
				"?," + // event_description
				"?," + // event_domain_id
				editor.getObmId() + // event_usercreate
				",?," + // origin
				"?," + // type
				"now())"; // event_timecreate

		PreparedStatement ps = con.prepareStatement(evQ);
		fillEventStatement(ps, ev, editor, 1);
		ps.executeUpdate();
		int id = ObmHelper.lastInsertId(con);
		ev.setUid("" + id);
		ev.setDatabaseId(id);

		ps.close();

		LinkedEntity le = ObmHelper.linkEntity(con, "EventEntity", "event_id",
				id);
		ev.setEntityId(le.getEntityId());

		insertAttendees(editor, id, con, ev.getAttendees());

		insertExceptions(editor, ev, con, id);

		insertEventExceptions(editor, ev, con, id);

		Integer a = ev.getAlert();
		if (a != null && a >= 0) {
			ps = con
					.prepareStatement("insert into EventAlert "
							+ "(eventalert_event_id, eventalert_user_id, eventalert_duration, eventalert_usercreate) "
							+ "values (?, ?, ?, ?)");
			ps.setInt(1, id);
			ps.setInt(2, editor.getObmId());
			ps.setInt(3, a);
			ps.setInt(4, editor.getObmId());
			ps.executeUpdate();
		}
		if (ps != null) {
			ps.close();
			ps = null;
		}
		return ev;
	}

	private void insertEventExceptions(AccessToken editor, Event ev,
			Connection con, int id) throws SQLException {
		if (ev.getRecurrence() != null) {
			for (Event evExcept : ev.getRecurrence().getEventExceptions()) {
				evExcept.setParentId(id);
				createEvent(con, editor, evExcept);
			}

		}

	}

	private void insertExceptions(AccessToken editor, Event ev, Connection con,
			int id) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("insert into EventException "
							+ "(eventexception_event_id, eventexception_date, eventexception_usercreate) "
							+ "values (?, ?, " + editor.getObmId() + ")");
			for (Date exd : ev.getRecurrence().getExceptions()) {
				ps.setInt(1, id);
				ps.setTimestamp(2, new Timestamp(exd.getTime()));
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}
	}

	private Event eventFromCursor(Calendar cal, ResultSet evrs)
			throws SQLException {
		Event e = new Event();
		int id = evrs.getInt("event_id");
		e.setUid("" + id);
		e.setDatabaseId(id);
		e.setTimeUpdate(evrs.getTime("event_timeupdate"));
		e.setType(EventType.valueOf(evrs.getString("event_type")));
		e.setParentId(evrs.getInt("event_parent_id"));
		e.setExtId(evrs.getString("event_ext_id"));
		e.setOpacity(EventOpacity.valueOf(evrs.getString("event_opacity")));
		e.setCategory(evrs.getString("eventcategory1_label")); // cat as string
		// in
		// sync ??
		e.setTitle(evrs.getString("event_title"));
		e.setLocation(evrs.getString("event_location"));
		cal.setTimeInMillis(evrs.getTimestamp("event_date").getTime());
		e.setDate(cal.getTime());
		e.setDuration(evrs.getInt("event_duration"));
		e.setPriority(evrs.getInt("event_priority"));
		e.setPrivacy(evrs.getInt("event_privacy"));
		e.setAllday(evrs.getBoolean("event_allday"));
		e.setDescription(evrs.getString("event_description"));

		EventRecurrence er = new EventRecurrence();
		er.setKind(RecurrenceKind.valueOf(evrs.getString("event_repeatkind")));
		er.setDays(evrs.getString("event_repeatdays"));
		er.setFrequence(evrs.getInt("event_repeatfrequence"));
		er.setEnd(evrs.getTimestamp("event_endrepeat"));
		e.setRecurrence(er);

		e.setEntityId(evrs.getInt("evententity_entity_id"));
		e.setOwner(evrs.getString("userobm_login"));
		e.setOwnerEmail(evrs.getString("userobm_login") + "@"
				+ evrs.getString("domain_name"));
		return e;
	}

	private int fillEventStatement(PreparedStatement ps, Event ev,
			AccessToken at, int i) throws SQLException {
		int idx = i;
		if (ev.getParentId() > 0) {
			ps.setInt(idx++, ev.getParentId());
		} else {
			ps.setNull(idx++, Types.INTEGER);
		}
		ps.setString(idx++, ev.getExtId());
		ps.setString(idx++, "Europe/Paris");
		ps.setObject(idx++, ev.getOpacity().getJdbcObject(ObmHelper.getType()));
		ps.setString(idx++, ev.getTitle());
		ps.setString(idx++, ev.getLocation());
		Integer cat = catIdFromString(ps.getConnection(), ev.getCategory(), at
				.getDomainId());
		if (cat != null) {
			ps.setInt(idx++, cat);
		} else {
			ps.setNull(idx++, Types.INTEGER);
		}
		ps.setInt(idx++, ev.getPriority());
		ps.setInt(idx++, ev.getPrivacy());
		ps.setTimestamp(idx++, new Timestamp(ev.getDate().getTime()));
		ps.setInt(idx++, ev.getDuration());
		ps.setBoolean(idx++, ev.isAllday());
		EventRecurrence r = ev.getRecurrence();
		ps.setString(idx++, r.getKind().toString());
		ps.setInt(idx++, r.getFrequence());
		ps.setString(idx++, r.getDays());
		if (r.getEnd() != null) {
			ps.setTimestamp(idx++, new Timestamp(r.getEnd().getTime()));
		} else {
			ps.setNull(idx++, Types.DATE);
		}
		ps.setNull(idx++, Types.VARCHAR); // color
		ps.setNull(idx++, Types.DATE); // FIXME completed
		ps.setNull(idx++, Types.VARCHAR); // FIXME url
		ps.setString(idx++, ev.getDescription());
		ps.setInt(idx++, at.getDomainId());
		ps.setString(idx++, at.getOrigin());
		ps.setObject(idx++, ev.getType().getJdbcObject(ObmHelper.getType()));
		return idx;
	}

	private String[] findDeletedEvents(ObmUser calendarUser, Date d) {
		List<Integer> result = new LinkedList<Integer>();
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		String q = "SELECT deletedevent_event_id FROM DeletedEvent WHERE "
				+ "deletedevent_user_id=? ";
		if (d != null) {
			q += "AND deletedevent_timestamp >= ?";
		}
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);
			ps.setInt(1, calendarUser.getUid());
			if (d != null) {
				ps.setTimestamp(2, new Timestamp(d.getTime()));
			}

			rs = ps.executeQuery();
			while (rs.next()) {
				result.add(rs.getInt(1));
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		String[] ret = new String[result.size()];
		int i = 0;
		for (Integer eid : result) {
			ret[i++] = (eid.toString());
		}
		return ret;
	}

	/**
	 * @param uid
	 * @param calendar
	 * @return
	 * @throws FindException
	 */
	public Event findEvent(AccessToken token, String uid) {
		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "INNER JOIN Domain ON event_domain_id=domain_id "
				+ "INNER JOIN EventEntity ON evententity_event_id=event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "WHERE event_id=? ";

		int id = Integer.parseInt(uid);

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		List<Event> changedEvent = new LinkedList<Event>();
		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		Event ret = null;
		String domainName = null;
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(ev);
			evps.setInt(1, id);
			evrs = evps.executeQuery();
			if (evrs.next()) {
				ret = eventFromCursor(cal, evrs);
				domainName = evrs.getString("domain_name");
				eventById.put(ret.getDatabaseId(), ret);
				changedEvent.add(ret);
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(changedEvent);
		loadAttendeesAndAlerts(token, eventById, evIdList, domainName);
		loadExceptions(cal, eventById, evIdList);
		loadEventExceptions(token, eventById, evIdList);
		return ret;
	}

	/**
	 * Return id of events who have : - start date, subject and duration if not
	 * allday equals to the given event - start date day, subject if allday
	 * equals to the given event
	 * 
	 * @param calendar
	 * @param event
	 * @return
	 */
	public List<String> findEventTwinKeys(String calendar, Event event,
			ObmDomain domain) {

		String squery = "SELECT DISTINCT event_id from Event "
				+ "INNER JOIN EventLink ON eventlink_event_id=event_id "
				+ "INNER JOIN UserEntity ON userentity_entity_id=eventlink_entity_id "
				+ "INNER JOIN UserObm ON userobm_id=userentity_user_id "
				+ "WHERE event_title=? AND userobm_login=? ";
		if (event.isAllday()) {
			squery += "AND event_date >= ? AND event_date <= ? AND (event_duration >= 86400 OR event_allday) ";
		} else {
			squery += "AND event_date=? AND event_duration=? ";
		}
		squery += "AND userobm_domain_id=?";

		PreparedStatement query = null;
		Connection con = null;
		ResultSet rs = null;
		List<String> ret = new LinkedList<String>();

		try {
			con = ObmHelper.getConnection();
			query = con.prepareStatement(squery);
			int index = 1;
			query.setString(index++, event.getTitle());
			query.setString(index++, calendar);
			if (event.isAllday()) {

				java.util.Calendar dayAfter = java.util.Calendar.getInstance();
				dayAfter.setTimeZone(TimeZone.getTimeZone("GMT"));
				dayAfter.setTimeInMillis(event.getDate().getTime());
				dayAfter.add(java.util.Calendar.HOUR_OF_DAY, 11);

				java.util.Calendar dayBefore = java.util.Calendar.getInstance();
				dayBefore.setTimeZone(TimeZone.getTimeZone("GMT"));
				dayBefore.setTimeInMillis(event.getDate().getTime());
				dayBefore.add(java.util.Calendar.HOUR_OF_DAY, -11);

				query.setTimestamp(index++,  new java.sql.Timestamp(dayBefore
						.getTimeInMillis()));

				query.setTimestamp(index++,  new java.sql.Timestamp(dayAfter
						.getTimeInMillis()));
			} else {
				logger.info("Event.getDate() : " + event.getDate());
				query.setTimestamp(index++, new java.sql.Timestamp(event
						.getDate().getTime()));
				query.setInt(index++, event.getDuration());
			}

			if (domain != null) {
				query.setInt(index++, domain.getId());
			}
			rs = query.executeQuery();
			while (rs.next()) {
				ret.add(rs.getString(1));
			}
			logger.info("Found " + ret.size() + " results with title "
					+ event.getTitle() + " date: " + event.getDate()
					+ " duration: " + event.getDuration() + " domain_id: "
					+ domain.getId());
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, query, rs);
		}
		return ret;
	}

	public List<String> findRefusedEvents(ObmUser calendarUser, Date d) {
		List<Integer> result = new LinkedList<Integer>();
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		String q = "SELECT eventlink_event_id FROM EventLink, UserEntity WHERE "
				+ "eventlink_state='DECLINED' AND eventlink_entity_id=userentity_entity_id AND userentity_user_id=? ";
		if (d != null) {
			q += "AND eventlink_timeupdate >= ?";
		}
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);
			ps.setInt(1, calendarUser.getUid());
			if (d != null) {
				ps.setTimestamp(2, new Timestamp(d.getTime()));
			}

			rs = ps.executeQuery();
			while (rs.next()) {
				result.add(rs.getInt(1));
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		List<String> ret = new ArrayList<String>(result.size());
		for (Integer eid : result) {
			ret.add(eid.toString());
		}
		return ret;
	}

	private String getAttendeeDisplayName(ResultSet rs) throws SQLException {
		String last = rs.getString("userobm_lastname");
		String first = rs.getString("userobm_firstname");
		return last + (first != null ? " " + first : "");
	}

	private String getAttendeeEmail(ResultSet rs, String domainName)
			throws SQLException {
		String[] alias = rs.getString("userobm_email").split("\r\n");
		if (!alias[0].contains("@")) {
			return alias[0] + "@" + domainName;
		} else {
			return alias[0];
		}
	}

	private ParticipationRole getAttendeeRequired(ResultSet rs)
			throws SQLException {
		return ParticipationRole.valueOf(rs.getString("eventlink_required"));
	}

	private ParticipationState getAttendeeState(ResultSet rs)
			throws SQLException {
		return ParticipationState.getValueOf(rs.getString("eventlink_state"));
	}

	private int getAttendeePercent(ResultSet rs) throws SQLException {
		return rs.getInt("eventlink_percent");
	}

	/**
	 * Get freebusy for an user in the period dstart - dend
	 * 
	 * @param login
	 * @param dstart
	 * @param dend
	 * @param domain
	 * @return
	 */
	public String[] getFreeBusy(String login, Date dstart, Date dend,
			ObmDomain domain) {
		logger.info("freebusy " + login + " dstart: " + dstart + " dend: "
				+ dend);

		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "INNER JOIN EventLink ON eventlink_event_id=event_id "
				+ "INNER JOIN EventEntity ON evententity_event_id=event_id "
				+ "INNER JOIN UserEntity ON userentity_entity_id=eventlink_entity_id "
				+ "INNER JOIN UserObm ON userobm_id=userentity_user_id "
				+ "INNER JOIN Domain ON e.event_domain_id=domain_id "
				+ "LEFT JOIN EventCategory1 ON eventcategory1_id=event_category1_id "
				+ "WHERE "
				+ "userentity_user_id IN (select userobm_id from UserObm where userobm_login=? and userobm_domain_id=?) AND "
				+ "event_date >= ? AND event_date <= ? AND "
				+ "eventlink_state=?";

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		List<Event> events = new LinkedList<Event>();
		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(ev);
			evps.setString(1, login);
			evps.setInt(2, domain.getId());
			evps.setTimestamp(3, new Timestamp(dstart.getTime()));
			evps.setTimestamp(4, new Timestamp(dend.getTime()));
			evps.setObject(5, ParticipationState.ACCEPTED
					.getJdbcObject(ObmHelper.getType()));
			evrs = evps.executeQuery();
			while (evrs.next()) {
				Event e = eventFromCursor(cal, evrs);
				eventById.put(e.getDatabaseId(), e);
				events.add(e);
			}

		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(events);
		loadExceptions(cal, eventById, evIdList);

		String[] ret = new String[events.size()];
		Calendar ctemp = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		int i = 0;
		logger.info("freebusy found " + events.size() + " events.");
		for (Event evt : events) {
			Date dtemp = evt.getDate();
			StringBuilder line = new StringBuilder("FREEBUSY:");
			line.append(Helper.getUTCFormat(dtemp));
			line.append("/");
			ctemp.setTime(dtemp);
			ctemp.add(java.util.Calendar.SECOND, evt.getDuration());
			dtemp = ctemp.getTime();
			line.append(Helper.getUTCFormat(dtemp));
			line.append("\r\n");
			ret[i++] = line.toString();
		}

		return ret;
	}

	public EventChanges getSync(AccessToken token, ObmUser calendarUser,
			Date lastSync, EventType et) {
		EventChanges ret = new EventChanges();

		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "INNER JOIN EventLink att ON att.eventlink_event_id=e.event_id "
				+ "INNER JOIN UserEntity ue ON att.eventlink_entity_id=ue.userentity_entity_id "
				+ "INNER JOIN EventEntity ON e.event_id=evententity_event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "INNER JOIN Domain ON e.event_domain_id=domain_id "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "WHERE e.event_type=? AND ue.userentity_user_id=?";

		if (lastSync != null) {
			ev += " AND (e.event_timecreate >= ? OR att.eventlink_timecreate >= ? "
					+ "OR e.event_timeupdate >= ? OR att.eventlink_timeupdate >= ?)";
		}
		StringBuilder sb = new StringBuilder(ev);
		SyncRange sr = addSyncRanges(sb, token);

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		List<Event> changedEvent = new LinkedList<Event>();
		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(sb.toString());
			int idx = 1;
			evps.setObject(idx++, et.getJdbcObject(ObmHelper.getType()));
			evps.setObject(idx++, calendarUser.getUid());
			if (lastSync != null) {
				evps.setTimestamp(idx++, new Timestamp(lastSync.getTime()));
				evps.setTimestamp(idx++, new Timestamp(lastSync.getTime()));
				evps.setTimestamp(idx++, new Timestamp(lastSync.getTime()));
				evps.setTimestamp(idx++, new Timestamp(lastSync.getTime()));
			}
			if (sr != null) {
				evps.setTimestamp(idx++, sr.before);
				evps.setTimestamp(idx++, sr.after);
			}
			evrs = evps.executeQuery();
			boolean lastSyncSet = false;
			while (evrs.next()) {
				if (!lastSyncSet) {
					ret.setLastSync(evrs.getTimestamp("last_sync"));
					lastSyncSet = true;
				}

				Event e = eventFromCursor(cal, evrs);
				eventById.put(e.getDatabaseId(), e);
				changedEvent.add(e);
			}
			if (!lastSyncSet) {
				ret.setLastSync(ObmHelper.selectNow(con));
			}

		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(changedEvent);
		loadAttendeesAndAlerts(token, eventById, evIdList, calendarUser
				.getDomain().getName());
		loadExceptions(cal, eventById, evIdList);
		loadEventExceptions(token, eventById, evIdList);
		ret.setUpdated(changedEvent.toArray(new Event[changedEvent.size()]));
		ret.setRemoved(findDeletedEvents(calendarUser, lastSync));

		return ret;
	}

	private SyncRange addSyncRanges(StringBuilder sb, AccessToken token) {
		// TODO Auto-generated method stub
		String min = token.getServiceProperty("funis/sync_days_min");
		String max = token.getServiceProperty("funis/sync_days_max");
		if (min == null || max == null) {
			return null;
		}
		int minDays = Integer.parseInt(min);
		int maxDays = Integer.parseInt(max);
		Calendar now = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		now.add(Calendar.DAY_OF_MONTH, 0 - minDays);
		Timestamp before = new Timestamp(now.getTimeInMillis());
		now.add(Calendar.DAY_OF_MONTH, minDays + maxDays);
		Timestamp after = new Timestamp(now.getTimeInMillis());
		sb.append("AND (");
		sb.append("event_repeatkind != 'none' OR ");
		sb.append("(event_date >= ? AND event_date <= ?)");
		sb.append(")");
		logger.info(token.getUser() + " will use the sync range [ "
				+ before.toString() + " - " + after.toString() + " ]");
		return new SyncRange(before, after);
	}

	private Set<CalendarInfo> listUserRights(Set<CalendarInfo> l, ObmUser user)
			throws FindException {
		String query =
		// direct rights
		"select userobm_login, userobm_firstname, userobm_lastname, userobm_email, entityright_read, entityright_write "
				+ "from EntityRight "
				+ "inner join UserEntity u1 on entityright_consumer_id=u1.userentity_entity_id "
				+ "inner join CalendarEntity u2 on u2.calendarentity_entity_id=entityright_entity_id "
				+ "inner join UserObm on userobm_id=u2.calendarentity_calendar_id "
				+ "where u1.userentity_user_id=? and (entityright_read=1 or entityright_write=1) and userobm_email is not null "
				+ "and userobm_email != '' and userobm_archive != 1"
				+ " union "
				// public cals
				+ "select userobm_login, userobm_firstname, userobm_lastname, userobm_email, entityright_read, entityright_write "
				+ "from EntityRight "
				+ "inner join CalendarEntity u2 on u2.calendarentity_entity_id=entityright_entity_id "
				+ "inner join UserObm on userobm_id=u2.calendarentity_calendar_id "
				+ "where entityright_consumer_id is null and (entityright_read=1 or entityright_write=1) and userobm_email is not null and userobm_email != '' "
				+ " and userobm_domain_id="
				+ user.getDomain().getId()
				+ " and userobm_archive != 1";

		Connection con = null;
		ResultSet rs = null;
		PreparedStatement ps = null;

		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(query);
			ps.setInt(1, user.getUid());

			rs = ps.executeQuery();
			while (rs.next()) {
				processRightsRow(rs, l, user);
			}
		} catch (SQLException e) {
			logger.error("Error finding user rights", e);
			throw new FindException(e);
		} finally {
			try {
				ObmHelper.cleanup(con, ps, rs);
			} catch (Exception e) {
				logger.error("Could not clean up jdbc stuff");
			}
		}
		return l;
	}

	private Set<CalendarInfo> listGroupRights(Set<CalendarInfo> l, ObmUser user)
			throws FindException {
		String query = "select userobm_login, userobm_firstname, userobm_lastname, userobm_email, entityright_read, entityright_write "
				+ "from EntityRight "
				+ "inner join GroupEntity u1 on entityright_consumer_id=u1.groupentity_entity_id "
				+ "inner join CalendarEntity u2 on u2.calendarentity_entity_id=entityright_entity_id "
				+ "inner join UserObm on userobm_id=u2.calendarentity_calendar_id "
				+ "inner join of_usergroup on of_usergroup_group_id = u1.groupentity_group_id "
				+ "where of_usergroup_user_id=? "
				+ "and userobm_email is not null and userobm_email != '' and userobm_archive != 1";

		Connection con = null;
		ResultSet rs = null;
		PreparedStatement ps = null;

		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(query);
			ps.setInt(1, user.getUid());

			rs = ps.executeQuery();
			while (rs.next()) {
				processRightsRow(rs, l, user);
			}
		} catch (SQLException e) {
			logger.error("Error finding user rights", e);
			throw new FindException(e);
		} finally {
			try {
				ObmHelper.cleanup(con, ps, rs);
			} catch (Exception e) {
				logger.error("Could not clean up jdbc stuff");
			}
		}

		return l;
	}

	public List<CalendarInfo> listCalendars(ObmUser user) throws FindException {
		Set<CalendarInfo> rights = new HashSet<CalendarInfo>();
		CalendarInfo myself = new CalendarInfo();
		myself.setMail(Helper.constructEmailFromList(user.getEmail(), user
				.getDomain().getName()));
		myself.setUid(user.getLogin());
		myself.setFirstname(user.getFirstName());
		myself.setLastname(user.getLastName());
		myself.setRead(true);
		myself.setWrite(true);
		rights.add(myself);

		listUserRights(rights, user);
		listGroupRights(rights, user);
		ArrayList<CalendarInfo> ret = new ArrayList<CalendarInfo>(rights.size());
		ret.addAll(rights);
		return ret;
	}

	private void processRightsRow(ResultSet rs, Set<CalendarInfo> l,
			ObmUser user) throws SQLException {
		String email = Helper.constructEmailFromList(rs.getString(4), user
				.getDomain().getName());
		CalendarInfo info = new CalendarInfo();
		info.setUid(rs.getString(1));
		info.setFirstname(rs.getString(2));
		info.setLastname(rs.getString(3));
		info.setMail(email);
		info.setRead(rs.getBoolean(5));
		info.setWrite(rs.getBoolean(6));

		if (info.isWrite()) {
			info.setRead(true);
		}
		if (!l.contains(info)) {
			l.add(info);
		} else {
			Map<String, CalendarInfo> uidIdx = new HashMap<String, CalendarInfo>();
			for (CalendarInfo ci : l) {
				uidIdx.put(ci.getUid(), ci);
			}
			CalendarInfo old = uidIdx.get(info.getUid());

			if (info.isWrite() && !old.isWrite()) {
				l.remove(old);
				l.add(info);
			}
		}
	}

	private void loadAttendeesAndAlerts(AccessToken token,
			Map<Integer, Event> eventById, String evIdList, String domainName) {
		if (eventById.isEmpty()) {
			return;
		}
		String attAlerts = "SELECT "
				+ ATT_AND_ALERT_FIELDS
				+ " FROM EventLink att "
				+ "LEFT JOIN EventAlert ON eventlink_event_id=eventalert_event_id "
				+ "INNER JOIN UserEntity ON att.eventlink_entity_id=userentity_entity_id "
				+ "INNER JOIN UserObm ON userobm_id=userentity_user_id "
				+ "WHERE eventlink_event_id IN (" + evIdList + ")";

		attAlerts = attAlerts // add contact attendees
				+ " UNION SELECT "
				+ CONTACT_AND_ALERT_FIELDS
				+ " FROM EventLink att "
				+ "INNER JOIN ContactEntity ON att.eventlink_entity_id=contactentity_entity_id "
				+ "INNER JOIN Contact ON contact_id=contactentity_contact_id "
				+ "INNER JOIN Email ON email_entity_id=contactentity_entity_id "
				+ "WHERE eventlink_event_id IN ("
				+ evIdList
				+ ")"
				+ " GROUP BY userobm_firstname, userobm_lastname , eventlink_event_id, "
				+ "eventlink_state, eventlink_required, eventlink_percent, eventalert_duration, contactentity_contact_id";

		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection con = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(attAlerts);
			rs = ps.executeQuery();
			while (rs.next()) {
				Event e = eventById.get(rs.getInt(1));
				Attendee att = new Attendee();
				att.setDisplayName(getAttendeeDisplayName(rs));
				att.setEmail(getAttendeeEmail(rs, domainName));
				att.setState(getAttendeeState(rs));
				att.setRequired(getAttendeeRequired(rs));
				att.setPercent(getAttendeePercent(rs));
				e.addAttendee(att);
				if (token != null) {
					int userId = rs.getInt("userentity_user_id");
					if (token.getObmId() == userId) {
						int alert = rs.getInt("eventalert_duration");
						if (!rs.wasNull() && alert >= 0) {
							e.setAlert(alert);
						}
					}
				}
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

	}

	private void loadEventExceptions(AccessToken token,
			Map<Integer, Event> eventById, String evIdList) {
		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "INNER JOIN Domain ON event_domain_id=domain_id "
				+ "INNER JOIN EventEntity ON evententity_event_id=event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "WHERE event_parent_id IN (" + evIdList + ") ";

		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection con = null;

		String domainName = null;

		Map<Integer, Event> evenExcepttById = new HashMap<Integer, Event>();
		List<Event> changedEvent = new LinkedList<Event>();
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {

			con = ObmHelper.getConnection();
			ps = con.prepareStatement(ev);
			rs = ps.executeQuery();

			while (rs.next()) {
				Event eventExcept = eventFromCursor(cal, rs);
				domainName = rs.getString("domain_name");
				Event event = eventById.get(eventExcept.getParentId());
				if (event != null) {
					event.getRecurrence().addEventException(eventExcept);
					changedEvent.add(eventExcept);
					evenExcepttById.put(eventExcept.getDatabaseId(),
							eventExcept);
				}
			}

		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		String evExceptIdList = buildEventId(changedEvent);
		loadAttendeesAndAlerts(token, evenExcepttById, evExceptIdList,
				domainName);
	}

	private void loadExceptions(Calendar cal, Map<Integer, Event> eventById,
			String evIdList) {
		if (eventById.isEmpty()) {
			return;
		}
		String exceps = "SELECT "
				+ EXCEPS_FIELDS
				+ " FROM Event e LEFT JOIN EventException on event_id=eventexception_event_id "
				+ "WHERE event_id IN (" + evIdList + ")";
		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection con = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(exceps);
			rs = ps.executeQuery();
			while (rs.next()) {
				Event e = eventById.get(rs.getInt(1));
				EventRecurrence er = e.getRecurrence();
				Timestamp t = rs.getTimestamp(2);
				if (t != null) {
					cal.setTimeInMillis(t.getTime());
					er.addException(cal.getTime());
				}
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
	}

	public Event markUpdated(Event ev) {
		Connection con = null;
		Statement st = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			st = con.createStatement();
			st
					.executeUpdate("UPDATE Event set event_id=event_id WHERE event_id="
							+ ev.getDatabaseId());
			ut.commit();
		} catch (Throwable se) {
			ObmHelper.rollback(ut);
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, st, null);
		}
		return ev;
	}

	/**
	 * @param calendar
	 * @param ev
	 * @param onlyUpdateMyself
	 * @param updateAttendees
	 * @return
	 */
	public Event modifyEvent(AccessToken at, Event ev,
			boolean onlyUpdateMyself, boolean updateAttendees) {
		Event old = findEvent(at, ev.getUid());
		if (old == null) {
			return null;
		}
		old.getAttendees().removeAll(ev.getAttendees());
		List<Attendee> attendeetoRemove = old.getAttendees();

		old.getRecurrence().getEventExceptions().removeAll(
				ev.getRecurrence().getEventExceptions());

		String upQ = "UPDATE Event SET " + "event_userupdate=?, "
				+ "event_type=?, " + "event_timezone=?, " + "event_opacity=?, "
				+ "event_title=?, " + "event_location=?, "
				+ "event_category1_id=?, " + "event_priority=?, "
				+ "event_privacy=?, " + "event_date=?, " + "event_duration=?, "
				+ "event_allday=?, " + "event_repeatkind=?, "
				+ "event_repeatfrequence=?, " + "event_repeatdays=?, "
				+ "event_endrepeat=?, " + "event_completed=?, "
				+ "event_url=?, " + "event_description=?, " + "event_origin=? "
				+ "WHERE event_id=?";

		logger.info("should modify event with title " + ev.getTitle()
				+ " date: " + ev.getDate() + " id: " + ev.getDatabaseId()
				+ " onlyUpdateMyself: " + onlyUpdateMyself);
		Connection con = null;
		PreparedStatement ps = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(upQ);
			ps.setInt(1, at.getObmId());
			ps.setObject(2, ev.getType().getJdbcObject(ObmHelper.getType()));
			ps.setString(3, "Europe/Paris");
			ps.setObject(4, ev.getOpacity().getJdbcObject(ObmHelper.getType()));
			ps.setString(5, ev.getTitle());
			ps.setString(6, ev.getLocation());
			Integer cat = catIdFromString(con, ev.getCategory(), at
					.getDomainId());
			if (cat != null) {
				ps.setInt(7, cat);
			} else {
				ps.setNull(7, Types.INTEGER);
			}
			ps.setInt(8, ev.getPriority());
			ps.setInt(9, ev.getPrivacy());
			ps.setTimestamp(10, new Timestamp(ev.getDate().getTime()));
			ps.setInt(11, ev.getDuration());
			ps.setBoolean(12, ev.isAllday());
			EventRecurrence er = ev.getRecurrence();
			ps.setString(13, er.getKind().toString());
			ps.setInt(14, er.getFrequence());
			ps.setString(15, er.getDays());
			if (er.getEnd() != null) {
				ps.setTimestamp(16, new Timestamp(er.getEnd().getTime()));
			} else {
				ps.setNull(16, Types.TIMESTAMP);
			}
			ps.setNull(17, Types.TIMESTAMP);
			ps.setNull(18, Types.VARCHAR);
			ps.setString(19, ev.getDescription());
			ps.setString(20, at.getOrigin());
			ps.setInt(21, ev.getDatabaseId());

			if (!onlyUpdateMyself) {
				ps.executeUpdate();
				ps.close();
				ps = null;
			}

			if (!onlyUpdateMyself && updateAttendees) {
				removeAttendees(at, con, attendeetoRemove, ev);
			}
			if (updateAttendees) {
				updateAttendees(at, con, ev, onlyUpdateMyself);
			}
			updateAlerts(at, con, ev, onlyUpdateMyself);
			
			if (!onlyUpdateMyself) {
				removeEventExceptions(at, con, ev.getDatabaseId());
			}
			updateEventException(at, con, ev, onlyUpdateMyself, updateAttendees);

			if (!onlyUpdateMyself) {
				updateExceptions(at, con, ev);
			}
			ut.commit();
		} catch (Throwable se) {
			ObmHelper.rollback(ut);
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, null);
		}

		int id = ev.getDatabaseId();

		return findEvent(at, "" + id);
	}

	private void updateEventException(AccessToken at, Connection con,
			Event eventParent, boolean onlyUpdateMyself, boolean updateAttendees)
			throws SQLException {
		if (eventParent.getRecurrence() != null) {
			for (Event event : eventParent.getRecurrence().getEventExceptions()) {
				event.setParentId(eventParent.getDatabaseId());
				if (event.getDatabaseId() == 0) {
					createEvent(con, at, event);
				} else {
					modifyEvent(at, event, onlyUpdateMyself, updateAttendees);
				}

			}
		}
	}

	private void removeEventExceptions(AccessToken at, Connection con,
			int eventParentId) throws SQLException {

		// disabled in 2.2

		// logger.info("event update will remove all eventException for event "+eventParentId+".");
		// String q = "delete from Event where event_parent_id=?  ";
		// PreparedStatement ps = null;
		// try {
		// ps = con.prepareStatement(q);
		// ps.setInt(1, eventParentId);
		// ps.executeUpdate();
		// } finally {
		// ObmHelper.cleanup(null, ps, null);
		// }

	}

	private void removeAttendees(AccessToken token, Connection con,
			List<Attendee> toRemove, Event ev) throws SQLException {
		logger.info("event update will remove " + toRemove.size()
				+ " attendees.");
		String q = "delete from EventLink where eventlink_event_id=? and eventlink_entity_id=? ";
		PreparedStatement ps = null;
		try {
			ps = con.prepareStatement(q);
			for (Attendee at : toRemove) {
				Integer id = UserManagement.getInstance()
						.userEntityFromEmailQuery(con, at.getEmail());
				if (id == null) {
					id = UserManagement.getInstance()
							.contactEntityFromEmailQuery(con, at.getEmail());
				}
				ps.setInt(1, ev.getDatabaseId());
				ps.setInt(2, id);
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

	}

	private void updateExceptions(AccessToken at, Connection con, Event ev)
			throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("delete from EventException where eventexception_event_id=?");
			ps.setInt(1, ev.getDatabaseId());
			ps.executeUpdate();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}
		insertExceptions(at, ev, con, ev.getDatabaseId());
	}

	/**
	 * @param calendar
	 * @param uid
	 * @return
	 */
	public Event removeEvent(AccessToken token, String uid) {
		Event ev = findEvent(token, uid);
		if (ev == null) {
			return null;
		}
		Connection con = null;
		PreparedStatement dev = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			dev = con
					.prepareStatement("INSERT INTO DeletedEvent (deletedevent_event_id, deletedevent_user_id, deletedevent_origin, deletedevent_timestamp) "
							+ "VALUES (?, ?, ?, now())");
			int databaseId = ev.getDatabaseId();
			UserManagement um = UserManagement.getInstance();
			for (Attendee at : ev.getAttendees()) {
				Integer userId = um.userIdFromEmailQuery(con, at.getEmail());
				if (userId != null) {
					dev.setInt(1, databaseId);
					dev.setInt(2, userId);
					dev.setString(3, token.getOrigin());
					dev.addBatch();
				}
			}
			dev.executeBatch();
			dev.close();

			dev = con.prepareStatement("DELETE FROM Event WHERE event_id=?");
			dev.setInt(1, databaseId);
			dev.executeUpdate();
			/*
			 * dev.close();
			 * 
			 * dev = con.prepareStatement(
			 * "UPDATE Event SET event_parent_id='0' WHERE event_parent_id=?");
			 * dev.setInt(1, databaseId); dev.executeUpdate();
			 */
			removeEventExceptions(token, con, databaseId);

			ut.commit();
		} catch (Throwable se) {
			ObmHelper.rollback(ut);
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, dev, null);
		}

		return ev;
	}

	private void insertAttendees(AccessToken editor, int id, Connection con,
			List<Attendee> attendees) throws SQLException {
		String attQ = "INSERT INTO EventLink (" + ATT_INSERT_FIELDS
				+ ") VALUES (" + "?, " + // event_id
				"?, " + // entity_id
				"?, " + // state
				"?, " + // required
				"?," + // percent
				"?" + // user_create
				")";
		PreparedStatement ps = null;

		try {
			ps = con.prepareStatement(attQ);
			for (Attendee at : attendees) {
				Integer entityId = UserManagement.getInstance()
						.userEntityFromEmailQuery(con, at.getEmail());
				if (entityId == null) {
					logger.info("user with email " + at.getEmail()
							+ " not found. Checking contacts.");
					entityId = UserManagement.getInstance()
							.contactEntityFromEmailQuery(con, at.getEmail());
					if (entityId == null) {
						logger.info("Attendee " + at.getEmail()
								+ " not found in OBM, will create a contact");
						Contact c = new Contact();
						c.setLastname(at.getDisplayName());
						c.setFirstname("");
						c.addEmail("INTERNET;X-OBM-Ref1", new Email(at
								.getEmail()));
						c.setCollected(true);
						c = ContactHome.getInstance().createContact(con,
								editor, c);
						entityId = c.getEntityId();
					}
				}

				ps.setInt(1, id);
				ps.setInt(2, entityId);
				ps.setObject(3, at.getState()
						.getJdbcObject(ObmHelper.getType()));
				ps.setObject(4, at.getRequired().getJdbcObject(
						ObmHelper.getType()));
				ps.setInt(5, at.getPercent());
				ps.setInt(6, editor.getObmId());

				ps.addBatch();
				logger.info("Adding attendee " + at.getEmail());
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}
	}

	private void updateAttendees(AccessToken updater, Connection con,
			Event ev, boolean onlyUpdateMyself) throws SQLException {
		String q = "update EventLink set eventlink_state=?, eventlink_required=?, eventlink_userupdate=?, eventlink_percent=? "
				+ "where eventlink_event_id=? AND "
				+ "eventlink_entity_id IN "
				+ "(select userentity_entity_id from UserEntity where userentity_user_id=?  "
				+ "UNION SELECT contactentity_entity_id from ContactEntity where contactentity_entity_id=?)";

		PreparedStatement ps = null;
		int[] updatedAttendees;
		List<Attendee> mightInsert = new LinkedList<Attendee>();
		List<Attendee> toInsert = new LinkedList<Attendee>();

		try {
			ps = con.prepareStatement(q);
			for (Attendee at : ev.getAttendees()) {
				Integer userId = UserManagement.getInstance()
						.userIdFromEmailQuery(con, at.getEmail());
				if (userId == null) {
					userId = UserManagement.getInstance()
							.contactEntityFromEmailQuery(con, at.getEmail());
				}
				if (userId == null) {
					logger.info("skipping attendee update for email "
							+ at.getEmail() + ". Will add as contact");
					toInsert.add(at);
					continue;
				}

				int idx = 1;
				ps.setObject(idx++, at.getState().getJdbcObject(
						ObmHelper.getType()));
				ps.setObject(idx++, at.getRequired().getJdbcObject(
						ObmHelper.getType()));
				ps.setInt(idx++, updater.getObmId());
				ps.setInt(idx++, at.getPercent());
				ps.setInt(idx++, ev.getDatabaseId());
				ps.setInt(idx++, userId);
				ps.setInt(idx++, userId);
				if (!onlyUpdateMyself
						|| (onlyUpdateMyself && updater.getEmail().equals(
								at.getEmail()))) {
					ps.addBatch();
					mightInsert.add(at);
				}
			}
			updatedAttendees = ps.executeBatch();

		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

		for (int i = 0; i < updatedAttendees.length; i++) {
			if (updatedAttendees[i] == 0) {
				Attendee at = mightInsert.get(i);
				toInsert.add(at);
			}
		}
		if (!onlyUpdateMyself) {
			logger.info("event modification needs to add " + toInsert.size()
					+ " attendees.");
			insertAttendees(updater, ev.getDatabaseId(), con, toInsert);
		}
		
		if (onlyUpdateMyself) {
			try {
				String query = " UPDATE Event SET event_userupdate=? where event_id = ?";
				ps =  con.prepareStatement(query);
				ps.setInt(1, updater.getObmId());
				ps.setInt(2, ev.getDatabaseId());
				ps.executeUpdate();
			} finally {
				ObmHelper.cleanup(null, ps, null);
			}
		}
		
		Statement st = null;
		try {
			st = con.createStatement();
			if (ev.getAlert() == null || ev.getAlert() < 0) {
				st
						.executeUpdate("delete from EventAlert where eventalert_user_id="
								+ updater.getObmId()
								+ " AND eventalert_event_id="
								+ ev.getDatabaseId());
			} else {
				int upd = st
						.executeUpdate("update EventAlert set eventalert_duration="
								+ ev.getAlert()
								+ ", eventalert_userupdate="
								+ updater.getObmId()
								+ " where eventalert_user_id="
								+ updater.getObmId()
								+ " AND eventalert_event_id="
								+ ev.getDatabaseId());
				if (upd <= 0) {
					st
							.executeUpdate("insert into EventAlert (eventalert_duration, eventalert_event_id, eventalert_usercreate, eventalert_user_id)"
									+ " values ("
									+ ev.getAlert()
									+ ","
									+ ev.getDatabaseId()
									+ ","
									+ updater.getObmId()
									+ ","
									+ updater.getObmId() + " )");
				}
			}
		} finally {
			ObmHelper.cleanup(null, st, null);
		}

	}

	private void updateAlerts(AccessToken updater, Connection con,
			Event ev, boolean onlyUpdateMyself) throws SQLException {

		Statement st = null;
		try {
			st = con.createStatement();
			if (ev.getAlert() == null || ev.getAlert() < 0) {
				st
						.executeUpdate("delete from EventAlert where eventalert_user_id="
								+ updater.getObmId()
								+ " AND eventalert_event_id="
								+ ev.getDatabaseId());
			} else {
				int upd = st
						.executeUpdate("update EventAlert set eventalert_duration="
								+ ev.getAlert()
								+ ", eventalert_userupdate="
								+ updater.getObmId()
								+ " where eventalert_user_id="
								+ updater.getObmId()
								+ " AND eventalert_event_id="
								+ ev.getDatabaseId());
				if (upd <= 0) {
					st
							.executeUpdate("insert into EventAlert (eventalert_duration, eventalert_event_id, eventalert_usercreate, eventalert_user_id)"
									+ " values ("
									+ ev.getAlert()
									+ ","
									+ ev.getDatabaseId()
									+ ","
									+ updater.getObmId()
									+ ","
									+ updater.getObmId() + " )");
				}
			}
		} finally {
			ObmHelper.cleanup(null, st, null);
		}

	}

	public Event findEventByExtId(AccessToken token, String extId) {
		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "INNER JOIN Domain ON event_domain_id=domain_id "
				+ "INNER JOIN EventEntity ON evententity_event_id=event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "WHERE event_ext_id=? ";

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		List<Event> changedEvent = new LinkedList<Event>();
		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		Event ret = null;
		String domainName = null;
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(ev);
			evps.setString(1, extId);
			evrs = evps.executeQuery();
			if (evrs.next()) {
				ret = eventFromCursor(cal, evrs);
				domainName = evrs.getString("domain_name");
				eventById.put(ret.getDatabaseId(), ret);
				changedEvent.add(ret);
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(changedEvent);
		loadAttendeesAndAlerts(token, eventById, evIdList, domainName);
		loadExceptions(cal, eventById, evIdList);
		loadEventExceptions(token, eventById, evIdList);
		return ret;
	}

	public List<Event> findListEventsFromIntervalDate(AccessToken token,
			ObmUser calendarUser, Date start, Date end, EventType et) {
		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "INNER JOIN EventLink att ON att.eventlink_event_id=e.event_id "
				+ "INNER JOIN UserEntity ue ON att.eventlink_entity_id=ue.userentity_entity_id "
				+ "INNER JOIN EventEntity ON e.event_id=evententity_event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "INNER JOIN Domain ON e.event_domain_id=domain_id "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "WHERE e.event_type=? AND ue.userentity_user_id=? "
				+ "AND ((event_repeatkind != 'none' AND (event_endrepeat IS NULL OR event_endrepeat >= ?)) OR "
				+ "(event_date >= ? AND event_date <= ?) )";

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		List<Event> changedEvent = new LinkedList<Event>();
		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		List<Event> ret = new LinkedList<Event>();
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(ev);
			int idx = 1;
			evps.setObject(idx++, et.getJdbcObject(ObmHelper.getType()));
			evps.setObject(idx++, calendarUser.getUid());
			evps.setTimestamp(idx++, new Timestamp(start.getTime()));
			evps.setTimestamp(idx++, new Timestamp(start.getTime()));
			evps.setTimestamp(idx++, new Timestamp(end.getTime()));
			evrs = evps.executeQuery();
			while (evrs.next()) {
				Event event = eventFromCursor(cal, evrs);
				if (Ical4jHelper.isInIntervalDate(event, start, end)) {
					eventById.put(event.getDatabaseId(), event);
					changedEvent.add(event);
					ret.add(event);
				}
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(changedEvent);
		loadAttendeesAndAlerts(token, eventById, evIdList, calendarUser
				.getDomain().getName());
		loadExceptions(cal, eventById, evIdList);
		loadEventExceptions(token, eventById, evIdList);
		return ret;
	}

	public List<Event> findAllEvents(AccessToken token, ObmUser calendarUser,
			EventType et) {
		String ev = "SELECT "
				+ EVENT_SELECT_FIELDS
				+ " FROM Event e "
				+ "INNER JOIN EventLink att ON att.eventlink_event_id=e.event_id "
				+ "INNER JOIN UserEntity ue ON att.eventlink_entity_id=ue.userentity_entity_id "
				+ "INNER JOIN EventEntity ON e.event_id=evententity_event_id "
				+ "INNER JOIN UserObm ON e.event_owner=userobm_id "
				+ "INNER JOIN Domain ON e.event_domain_id=domain_id "
				+ "LEFT JOIN EventCategory1 ON e.event_category1_id=eventcategory1_id "
				+ "WHERE ue.userentity_user_id=? and e.event_type=? ";

		StringBuilder sb = new StringBuilder(ev);
		SyncRange sr = addSyncRanges(sb, token);

		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		Map<Integer, Event> eventById = new HashMap<Integer, Event>();
		List<Event> ret = new LinkedList<Event>();
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(sb.toString());
			int idx = 1;
			evps.setObject(idx++, calendarUser.getUid());
			evps.setObject(idx++, et.getJdbcObject(ObmHelper.getType()));

			if (sr != null) {
				evps.setTimestamp(idx++, sr.before);
				evps.setTimestamp(idx++, sr.after);
			}
			evrs = evps.executeQuery();

			while (evrs.next()) {
				Event event = eventFromCursor(cal, evrs);
				eventById.put(event.getDatabaseId(), event);
				ret.add(event);
			}

		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}

		String evIdList = buildEventId(ret);
		loadAttendeesAndAlerts(token, eventById, evIdList, calendarUser
				.getDomain().getName());
		loadExceptions(cal, eventById, evIdList);
		loadEventExceptions(token, eventById, evIdList);
		return ret;
	}

	public List<EventTimeUpdate> findAllEventTimeUpdate(AccessToken token,
			ObmUser calendarUser, EventType et) {
		String ev = "SELECT event_id, event_ext_id, event_parent_id,event_timeupdate"
				+ " FROM Event e "
				+ "INNER JOIN EventLink att ON att.eventlink_event_id=e.event_id "
				+ "INNER JOIN UserEntity ue ON att.eventlink_entity_id=ue.userentity_entity_id "
				+ "WHERE ue.userentity_user_id=? and e.event_type=? ";

		StringBuilder sb = new StringBuilder(ev);
		SyncRange sr = addSyncRanges(sb, token);
		PreparedStatement evps = null;
		ResultSet evrs = null;
		Connection con = null;

		List<EventTimeUpdate> ret = new LinkedList<EventTimeUpdate>();
		try {
			con = ObmHelper.getConnection();
			evps = con.prepareStatement(sb.toString());
			int idx = 1;
			evps.setObject(idx++, calendarUser.getUid());
			evps.setObject(idx++, et.getJdbcObject(ObmHelper.getType()));

			if (sr != null) {
				evps.setTimestamp(idx++, sr.before);
				evps.setTimestamp(idx++, sr.after);
			}
			evrs = evps.executeQuery();
			while (evrs.next()) {
				EventTimeUpdate e = new EventTimeUpdate();
				e.setUid("" + evrs.getInt("event_id"));
				e.setParentId(evrs.getInt("event_parent_id"));
				e.setTimeUpdate(evrs.getTime("event_timeupdate"));
				e.setExtId(evrs.getString("event_ext_id"));
				ret.add(e);
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, evps, evrs);
		}
		return ret;
	}
}
