/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.calendar;

import java.util.Date;
import java.util.List;

import org.obm.sync.auth.AccessToken;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.items.EventChanges;

import fr.aliacom.obm.common.FindException;
import fr.aliacom.obm.common.ObmDomain;
import fr.aliacom.obm.common.ObmUser;

/**
 * Calendar access methods
 * 
 */
public class CalendarService {

	private static final CalendarService instance = new CalendarService();

	/**
	 * @return the server instance
	 */
	public static CalendarService getInstance() {
		return instance;
	}

	private CalendarService() {
	}

	/**
	 * @param uid
	 * @param calendar
	 * @return
	 * @throws FindException
	 */
	public Event findEvent(AccessToken token, String uid, String calendar) throws FindException {
		return CalendarHome.getInstance().findEvent(token, uid);
	}

	/**
	 * @param calendar
	 * @param ev
	 * @param onlyUpdateMyself 
	 * @param updateAttendees 
	 * @return
	 */
	public Event modifyEvent(AccessToken at, Event ev, boolean onlyUpdateMyself, boolean updateAttendees) {
		return CalendarHome.getInstance().modifyEvent(at, ev, onlyUpdateMyself, updateAttendees);
	}

	/**
	 * @param calendar
	 * @param uid
	 * @return
	 */
	public Event removeEvent(AccessToken token, String calendar, String uid) {
		return CalendarHome.getInstance().removeEvent(token, uid);
	}

	/**
	 * @param ev
	 * @return
	 */
	public Event createEvent(AccessToken at, Event ev) {
		return CalendarHome.getInstance().createEvent(at, ev);
	}

	/**
	 * @param calendar
	 * @param lastSync
	 * @param type 
	 * @return
	 */
	public EventChanges getSync(AccessToken token, ObmUser calendarUser, Date lastSync, EventType type) {
		return CalendarHome.getInstance().getSync(token, calendarUser, lastSync,
				type);
	}

	public List<CalendarInfo> listCalendars(ObmUser user) throws FindException {
		return CalendarHome.getInstance().listCalendars(user);
	}

	/**
	 * @param uid
	 * @param calendar
	 * @return
	 * @throws FindException
	 */
	public Event markEventAsUpdated(Event ev) throws FindException {
		return CalendarHome.getInstance().markUpdated(ev);
	}

	public List<String> findEventTwinKeys(String calendar, Event event,
			ObmDomain domain) {
		return CalendarHome.getInstance().findEventTwinKeys(calendar, event,
				domain);
	}

	public List<String> findRefusedEventsKeys(ObmUser calendarUser, Date d) {
		return CalendarHome.getInstance().findRefusedEvents(calendarUser, d);
	}

	public String[] getFreeBusy(String login, Date dstart, Date dend,
			ObmDomain domain) {
		return CalendarHome.getInstance().getFreeBusy(login, dstart, dend,
				domain);
	}

	public Event findEventByExtId(AccessToken token, String extId,
			String calendar) {
		return CalendarHome.getInstance().findEventByExtId(token, extId);
	}

	public List<Event> findListEventsFromIntervalDate(AccessToken token, ObmUser calendarUser,
			Date start, Date end, EventType type) {
		return CalendarHome.getInstance().findListEventsFromIntervalDate(token, calendarUser, start, end, type);
	}

	public List<Event> findAllEvents(AccessToken token, ObmUser calendarUser, EventType et) {
		return CalendarHome.getInstance().findAllEvents(token, calendarUser, et);
	}

	public List<EventTimeUpdate> findAllEventTimeUpdate(AccessToken token,
			ObmUser calendarUser, EventType et) {
		return CalendarHome.getInstance().findAllEventTimeUpdate(token, calendarUser, et);
	}

}