/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.contact;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.base.KeyList;
import org.obm.sync.book.BookType;
import org.obm.sync.book.Contact;
import org.obm.sync.items.ContactChanges;
import org.obm.sync.services.IAddressBook;

import fr.aliacom.obm.common.UserManagement;

/**
 * OBM {@link IAddressBook} web service implementation
 * 
 * @author tom
 * 
 */
public class AddressBookBindingImpl implements IAddressBook {

	private static final Log logger = LogFactory
			.getLog(AddressBookBindingImpl.class);

	private UserManagement um;

	public AddressBookBindingImpl() {
		this.um = UserManagement.getInstance();
	}

	public boolean isReadOnly(AccessToken token, BookType book)
			throws AuthFault, ServerFault {
		if (book == BookType.contacts) {
			return false;
		} else {
			return true;
		}
	}

	public BookType[] listBooks(AccessToken token) {
		return new BookType[] { BookType.contacts, BookType.users };
	}

	public ContactChanges getSync(AccessToken token, BookType book, Date d)
			throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			logger.info("[" + token.getUser() + "] AddressBook : getSync(" + d
					+ ")");

			Set<ContactPrivacy> toSync = new HashSet<ContactPrivacy>();
			toSync.add(ContactPrivacy.PRIVATE);
			ContactChanges sync = AddressBookService.getInstance().getSync(
					book, d, token, toSync);

			return sync;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault("error find contacts ");
		}

	}

	public Contact createContact(AccessToken token, BookType book,
			Contact contact) throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			Contact c = AddressBookService.getInstance().createContact(token,
					book, contact);

			logger.info("[" + token.getUser() + "] AdrressBook : contact["
					+ c.getFirstname() + " " + c.getLastname() + "] created");
			return c;

		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public Contact createContactWithoutDuplicate(AccessToken token,
			BookType book, Contact contact) throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			contact.setUid(null);

			List<String> duplicates = AddressBookService.getInstance()
					.getContactTwinKeys(token, book, contact);
			if (duplicates != null && duplicates.size() > 0) {
				logger.info("[" + token.getUser() + "] AddressBook : "
						+ duplicates.size()
						+ " duplicate(s) found for contact ["
						+ contact.getFirstname() + "," + contact.getLastname()
						+ "]");
				return AddressBookService.getInstance().getContactFromId(token,
						book, Integer.parseInt(duplicates.get(0)));
			}

			Contact c = AddressBookService.getInstance().createContact(token,
					book, contact);

			logger.info("[" + token.getUser() + "] AddressBook : contact["
					+ c.getFirstname() + " " + c.getLastname() + "] created");
			return c;

		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public Contact modifyContact(AccessToken token, BookType book,
			Contact contact) throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			Contact c = AddressBookService.getInstance().modifyContact(token, book,
					contact);

			logger.info("[" + token.getUser() + "] contact[" + c.getFirstname()
					+ " " + c.getLastname() + "] modified");
			return c;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public Contact removeContact(AccessToken token, BookType book, String uid)
			throws AuthFault, ServerFault {
		um.checkToken(token);

		for (char c : uid.toCharArray()) {
			if (!Character.isDigit(c)) {
				return null;
			}
		}
		
		try {
			
			Contact c = AddressBookService.getInstance().removeContact(token,
					book, Integer.parseInt(uid));
			logger.info("[" + token.getUser() + "] contact[" + uid
					+ "] removed (archived)");
			return c;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public Contact getContactFromId(AccessToken token, BookType book, String id)
			throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			return AddressBookService.getInstance().getContactFromId(token,
					book, Integer.parseInt(id));
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public KeyList getContactTwinKeys(AccessToken token, BookType book,
			Contact contact) throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			List<String> keys = AddressBookService.getInstance()
					.getContactTwinKeys(token, book, contact);
			return new KeyList(keys, null);

		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	@Override
	public List<Contact> searchContact(AccessToken token, String query, int limit) throws AuthFault, ServerFault {
		um.checkToken(token);
		
		try {
			return AddressBookService.getInstance().searchContact(token, query, limit);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

}
