/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.contact;

import java.sql.SQLException;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.book.BookType;
import org.obm.sync.book.Contact;
import org.obm.sync.items.ContactChanges;

import fr.aliacom.obm.common.FindException;
import fr.aliacom.obm.common.StoreException;

/**
 * The server implementation
 * 
 * @author tom
 * 
 */
public class AddressBookService {

	private static final AddressBookService os = new AddressBookService();

	/**
	 * @return the server instance
	 */
	public static AddressBookService getInstance() {
		return os;
	}

	@SuppressWarnings("unused")
	private static final Log logger = LogFactory
			.getLog(AddressBookService.class);

	private UserHome uh;
	private ContactHome ch;

	private AddressBookService() {
		this.uh = UserHome.getInstance();
		this.ch = ContactHome.getInstance();
	}

	public ContactChanges getSync(BookType book, Date timestamp,
			AccessToken token, Set<ContactPrivacy> toSync) throws FindException {
		ContactUpdates updated = new ContactUpdates();
		Set<Integer> todelete = new HashSet<Integer>();

		if (book == BookType.users) {
			updated = uh.findUpdatedUsers(timestamp, token);
			todelete = uh.findRemovalCandidates(timestamp, token);

		} else {
			ContactUpdates cu = ch.findUpdatedContacts(timestamp, token);
			updated.addAll(cu);
			updated.setLastSync(cu.getLastSync());
			todelete.addAll(ch.findRemovalCandidates(timestamp, token));
		}
		ContactChanges sync = new ContactChanges();
		sync.setLastSync(updated.getLastSync());
		sync.setUpdated(updated);
		sync.setRemoved(todelete);

		return sync;
	}

	public Contact modifyContact(AccessToken token, BookType book, Contact c)
			throws StoreException, SQLException {
		if (book == BookType.users) {
			throw new StoreException("users not writable");
		} else {
			Contact previous = ch.findContact(token, c.getUid());
			if (previous != null) {
				ContactMerger cm = new ContactMerger();
				cm.merge(previous, c);
			} else {
				logger.warn("previous version not found for c.uid: "+c.getUid()+" c.last: "+c.getLastname());
			}
			return ch.modifyContact(token, c);
		}
	}

	public Contact createContact(AccessToken at, BookType book, Contact c)
			throws StoreException {
		if (book == BookType.users) {
			throw new StoreException("users not writable");
		} else {
			return ch.createContact(at, c);
		}
	}

	public Contact removeContact(AccessToken at, BookType book, int uid)
			throws StoreException {
		if (book == BookType.users) {
			throw new StoreException("users not writable");
		} else {
			return ch.removeContact(at, uid);
		}
	}

	public Contact getContactFromId(AccessToken token, BookType book, int id) {
		if (book == BookType.users) {
			return null;
		} else {
			return ch.findContact(token, new Integer(id).intValue());

		}
	}

	public List<String> getContactTwinKeys(AccessToken token, BookType book,
			Contact contact) {
		if (book == BookType.users) {
			return null;
		} else {
			return ch.findContactTwinKeys(token, contact);
		}
	}

	public List<Contact> searchContact(AccessToken token, String query,
			int limit) {

		return ch.searchContact(token, query, limit);
	}

}
