/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.contact;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.transaction.UserTransaction;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.book.Address;
import org.obm.sync.book.Contact;
import org.obm.sync.book.Email;
import org.obm.sync.book.InstantMessagingId;
import org.obm.sync.book.Phone;
import org.obm.sync.book.Website;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventRecurrence;
import org.obm.sync.calendar.ParticipationRole;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.calendar.RecurrenceKind;

import fr.aliacom.obm.common.calendar.CalendarHome;
import fr.aliacom.obm.utils.LinkedEntity;
import fr.aliacom.obm.utils.ObmHelper;

/**
 * SQL queries for contact for sync
 * 
 * @author tom
 * 
 */
public class ContactHome {
	private static ContactHome instance = new ContactHome();

	private static final Log logger = LogFactory.getLog(ContactHome.class);

	private ContactHome() {
	}

	/**
	 * @return the home instance
	 */
	public static ContactHome getInstance() {
		return instance;
	}

	private static final String CONTACT_SELECT_FIELDS = "contact_id, contact_firstname, contact_lastname, contactentity_entity_id, "
			+ "contact_aka, contact_company, contact_title, contact_service, contact_birthday_id, "
			+ "contact_anniversary_id, contact_middlename, contact_suffix, contact_manager, contact_assistant, contact_spouse, "
			+ "contact_comment ";

	private static final String MY_GROUPS_QUERY = "SELECT groupentity_entity_id FROM of_usergroup "
			+ "INNER JOIN GroupEntity ON of_usergroup_group_id=groupentity_group_id WHERE of_usergroup_user_id=?";

	/**
	 * Return public contacts
	 * 
	 * @param timestamp
	 * @param domain
	 * @param user
	 * @return
	 */
	public ContactUpdates findUpdatedContacts(Date timestamp, AccessToken at) {

		String q = "SELECT "
				+ CONTACT_SELECT_FIELDS
				+ ", now() as last_sync FROM Contact "
				+ " INNER JOIN SynchedContact ON (synchedcontact_contact_id=contact_id AND synchedcontact_user_id="
				+ at.getObmId()
				+ ") "
				+ "INNER JOIN ContactEntity ON contactentity_contact_id=contact_id "
				+ "LEFT JOIN EntityRight urights ON "
				+ "(urights.entityright_entity_id=contactentity_entity_id AND "
				+ "urights.entityright_consumer_id=(select userentity_entity_id FROM UserEntity WHERE userentity_user_id=?)) "
				+ "LEFT JOIN EntityRight grights ON grights.entityright_entity_id=contactentity_entity_id "
				+ "AND grights.entityright_consumer_id IN ("
				+ MY_GROUPS_QUERY
				+ ") "
				+ "LEFT JOIN EntityRight prights ON prights.entityright_entity_id=contactentity_entity_id AND prights.entityright_consumer_id IS NULL "
				+ "WHERE "
				+ "((contact_privacy=? AND contact_archive != 1 AND contact_usercreate=?) OR "
				+ "(contact_privacy=? AND contact_archive != 1) OR "
				+ "(contact_archive != 1 AND (urights.entityright_read=1 OR grights.entityright_read=1 OR prights.entityright_read=1)))";

		q += " AND (contact_timeupdate >= ? OR contact_timecreate >= ? OR synchedcontact_timestamp >= ?)";

		int idx = 1;

		ContactUpdates upd = new ContactUpdates();
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);

			// userentity_user_id
			ps.setInt(idx++, at.getObmId());
			// my groups
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PRIVATE.asInt());
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PUBLIC.asInt());

			ps.setTimestamp(idx++, new Timestamp(timestamp.getTime()));
			ps.setTimestamp(idx++, new Timestamp(timestamp.getTime()));
			ps.setTimestamp(idx++, new Timestamp(timestamp.getTime()));
			rs = ps.executeQuery();
			boolean setLastSync = true;
			Map<Integer, Contact> entityContact = new HashMap<Integer, Contact>();
			while (rs.next()) {
				if (setLastSync) {
					setLastSync = false;
					upd.setLastSync(rs.getTimestamp("last_sync"));
				}
				Contact c = contactFromCursor(rs);
				entityContact.put(c.getEntityId(), c);
				upd.add(c);
			}
			rs.close();
			rs = null;

			if (setLastSync) {
				upd.setLastSync(ObmHelper.selectNow(con));
			}

			if (!entityContact.isEmpty()) {
				loadPhones(con, entityContact);
				loadIMIdentifiers(con, entityContact);
				loadWebsites(con, entityContact);
				loadAddresses(at, con, entityContact);
				loadEmails(con, entityContact);
				loadBirthday(con, entityContact);
			}

		} catch (Throwable se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		logger.info("returning " + upd.size() + " contact(s) updated");

		return upd;
	}

	private void loadBirthday(Connection con,
			Map<Integer, Contact> entityContact) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Set<Integer> bdayIds = new HashSet<Integer>();
		HashMap<Integer, Contact> eventIdMap = new HashMap<Integer, Contact>();
		for (Contact c : entityContact.values()) {
			if (c.getBirthdayId() != null) {
				bdayIds.add(c.getBirthdayId());
				eventIdMap.put(c.getBirthdayId(), c);
			}
		}
		if (bdayIds.isEmpty()) {
			return;
		}
		String q = "select event_id, event_date from Event where event_id IN ("
				+ buildIdList(bdayIds) + ")";
		try {
			ps = con.prepareStatement(q);
			rs = ps.executeQuery();
			while (rs.next()) {
				int evId = rs.getInt(1);
				Contact c = eventIdMap.get(evId);
				c.setBirthday(rs.getTimestamp(2));
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, ps, rs);
		}
	}

	private Contact contactFromCursor(ResultSet rs) throws SQLException {
		Contact c = new Contact();

		c.setUid(rs.getInt(1));
		c.setFirstname(rs.getString(2));
		c.setLastname(rs.getString(3));
		c.setEntityId(rs.getInt(4));
		c.setAka(rs.getString(5));
		c.setCompany(rs.getString(6));
		c.setTitle(rs.getString(7));
		c.setService(rs.getString(8));
		c.setBirthdayId(rs.getInt(9));

		// "contact_anniversary_id, contact_middlename, contact_suffix, contact_manager, contact_assistant, contact_spouse ";
		// post freeze fields
		c.setAnniversaryId(rs.getInt(10));
		c.setMiddlename(rs.getString(11));
		c.setSuffix(rs.getString(12));
		c.setManager(rs.getString(13));
		c.setAssistant(rs.getString(14));
		c.setSpouse(rs.getString(15));

		c.setComment(rs.getString(16));

		return c;
	}

	/**
	 * @param c
	 * @return
	 */
	public Contact createContact(Connection con, AccessToken at, Contact c) {
		try {

			if (c.getBirthday() != null) {
				Event e = CalendarHome.getInstance().createEvent(con, at,
						getBirthday(at, c));
				c.setBirthdayId(e.getDatabaseId());
			}

			int contactId = insertIntoContact(con, at, c);
			LinkedEntity le = ObmHelper.linkEntity(con, "ContactEntity",
					"contact_id", contactId);
			int entityId = le.getEntityId();
			c.setEntityId(entityId);

			createOrUpdatePhones(con, entityId, c.getPhones());
			createOrUpdateAddresses(con, entityId, c.getAddresses());
			createOrUpdateEmails(con, entityId, c.getEmails());
			createOrUpdateWebsites(con, entityId, c.getWebsites());
			createOrUpdateIMIdentifiers(con, entityId, c.getImIdentifiers());

			c.setUid(contactId);
		} catch (Throwable se) {
			logger.error(se.getMessage(), se);
		}
		return c;
	}

	public Contact createContact(AccessToken at, Contact c) {
		Connection con = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			createContact(con, at, c);
			ut.commit();
		} catch (Throwable e) {
			ObmHelper.rollback(ut);
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, null, null);
		}
		return c;
	}

	private Event getBirthday(AccessToken token, Contact c) {
		Event e = new Event();
		e.setTitle(displayName(c));
		e.setDate(c.getBirthday());
		e.setDuration(86400);
		e.setAllday(true);
		EventRecurrence rec = new EventRecurrence();
		rec.setDays("0000000");
		rec.setFrequence(1);
		rec.setKind(RecurrenceKind.yearly);
		rec.setEnd(null);
		e.setRecurrence(rec);
		e.setPrivacy(1);
		e.setPriority(1);
		Attendee at = new Attendee();
		at.setEmail(token.getEmail());
		at.setRequired(ParticipationRole.CHAIR);
		at.setState(ParticipationState.ACCEPTED);
		e.addAttendee(at);
		logger.info("inserting birthday with date " + c.getBirthday());
		return e;
	}

	private String displayName(Contact c) {
		StringBuilder b = new StringBuilder(255);
		if (c.getFirstname() != null) {
			b.append(c.getFirstname());
			b.append(" ");
		}
		if (c.getLastname() != null) {
			b.append(c.getLastname());
		}
		return b.toString();
	}

	private void createOrUpdateIMIdentifiers(Connection con, int entityId,
			Map<String, InstantMessagingId> imIdentifiers) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("DELETE FROM IM WHERE im_entity_id=? AND im_label IN ("
							+ lblList(imIdentifiers.keySet()) + ")");
			ps.setInt(1, entityId);
			ps.executeUpdate();

			ps.close();
			ps = con
					.prepareStatement("INSERT INTO IM (im_entity_id, im_label, im_protocol, im_address) "
							+ "VALUES (?, ?, ?, ?)");
			for (String s : imIdentifiers.keySet()) {
				ps.setInt(1, entityId);
				ps.setString(2, s);
				ps.setString(3, imIdentifiers.get(s).getProtocol());
				ps.setString(4, imIdentifiers.get(s).getId());
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

	}

	private void createOrUpdateWebsites(Connection con, int entityId,
			Map<String, Website> websites) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("DELETE FROM Website WHERE website_entity_id=? AND website_label IN ("
							+ lblList(websites.keySet()) + ")");
			ps.setInt(1, entityId);
			ps.executeUpdate();

			ps.close();
			ps = con
					.prepareStatement("INSERT INTO Website (website_entity_id, website_label, website_url) "
							+ "VALUES (?, ?, ?)");
			for (String s : websites.keySet()) {
				ps.setInt(1, entityId);
				ps.setString(2, s);
				ps.setString(3, websites.get(s).getUrl());
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

	}

	private void createOrUpdateAddresses(Connection con, int entityId,
			Map<String, Address> addresses) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("DELETE FROM Address WHERE address_entity_id=? and address_label IN ("
							+ lblList(addresses.keySet()) + ")");
			ps.setInt(1, entityId);
			ps.executeUpdate();

			ps.close();
			ps = con
					.prepareStatement("INSERT INTO Address (address_entity_id, address_label, "
							+ "address_street, address_zipcode, address_town, address_expresspostal, address_country, address_state) "
							+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
			for (String s : addresses.keySet()) {
				Address ad = addresses.get(s);
				ps.setInt(1, entityId);
				ps.setString(2, s);
				ps.setString(3, ad.getStreet());
				ps.setString(4, ad.getZipCode());
				ps.setString(5, ad.getTown());
				ps.setString(6, ad.getExpressPostal());
				ps.setString(7, getCountryIso3166(con, ad.getCountry()));
				ps.setString(8, ad.getState());
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

	}

	private String lblList(Set<String> keySet) {
		StringBuilder sb = new StringBuilder(keySet.size() * 20);
		sb.append("'_unused_'");
		for (String s : keySet) {
			sb.append(",'");
			sb.append(s);
			sb.append("'");
		}
		return sb.toString();
	}

	private void createOrUpdateEmails(Connection con, int entityId,
			Map<String, Email> emails) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("DELETE FROM Email WHERE email_entity_id=? AND email_label IN ("
							+ lblList(emails.keySet()) + ")");
			ps.setInt(1, entityId);
			ps.executeUpdate();

			ps.close();
			ps = con
					.prepareStatement("INSERT INTO Email (email_entity_id, email_label, email_address) "
							+ "VALUES (?, ?, ?)");
			for (String s : emails.keySet()) {
				ps.setInt(1, entityId);
				ps.setString(2, s);
				ps.setString(3, emails.get(s).getEmail());
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}
	}

	private void createOrUpdatePhones(Connection con, int entityId,
			Map<String, Phone> phones) throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("DELETE FROM Phone WHERE phone_entity_id=? and phone_label IN ("
							+ lblList(phones.keySet()) + ")");
			ps.setInt(1, entityId);
			ps.executeUpdate();

			ps.close();
			ps = con
					.prepareStatement("INSERT INTO Phone (phone_entity_id, phone_label, phone_number) "
							+ "VALUES (?, ?, ?)");
			for (String s : phones.keySet()) {
				ps.setInt(1, entityId);
				ps.setString(2, s);
				ps.setString(3, phones.get(s).getNumber());
				ps.addBatch();
			}
			ps.executeBatch();
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}

	}

	private int insertIntoContact(Connection con, AccessToken at, Contact c)
			throws SQLException {
		PreparedStatement ps = null;
		try {
			ps = con
					.prepareStatement("INSERT INTO Contact "
							+ " (contact_firstname, contact_lastname, contact_origin, contact_domain_id, contact_usercreate, "
							+ "contact_company, contact_aka, contact_service, contact_title, contact_birthday_id, "
							+ "contact_privacy, contact_timecreate, "
							+ "contact_suffix, contact_middlename, contact_manager, contact_spouse, contact_assistant, "
							+ "contact_collected) "
							+ " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, 1, now(), ?, ?, ?, ?, ?, ?) ");
			int idx = 1;
			ps.setString(idx++, c.getFirstname());
			ps.setString(idx++, c.getLastname());
			ps.setString(idx++, at.getOrigin());
			ps.setInt(idx++, at.getDomainId());
			ps.setInt(idx++, at.getObmId());

			ps.setString(idx++, c.getCompany());
			ps.setString(idx++, c.getAka());
			ps.setString(idx++, c.getService());
			ps.setString(idx++, c.getTitle());

			if (c.getBirthdayId() != null && c.getBirthdayId() > 0) {
				ps.setInt(idx++, c.getBirthdayId());
			} else {
				ps.setNull(idx++, Types.BIGINT);
			}

			ps.setString(idx++, c.getSuffix());
			ps.setString(idx++, c.getMiddlename());
			ps.setString(idx++, c.getManager());
			ps.setString(idx++, c.getSpouse());
			ps.setString(idx++, c.getAssistant());
			ps.setBoolean(idx++, c.isCollected());

			ps.executeUpdate();

			ps.close();
			ps = null;

			int contactId = ObmHelper.lastInsertId(con);

			ps = con
					.prepareStatement("INSERT INTO SynchedContact (synchedcontact_contact_id, synchedcontact_user_id) VALUES (?, ?)");
			ps.setInt(1, contactId);
			ps.setInt(2, at.getObmId());
			ps.executeUpdate();

			return contactId;
		} finally {
			ObmHelper.cleanup(null, ps, null);
		}
	}

	/**
	 * @param c
	 */
	public Contact modifyContact(AccessToken token, Contact c) {

		if (!hasRightsOn(token, c.getUid())) {
			logger.warn("contact " + c.getLastname() + " " + c.getFirstname()
					+ "(" + c.getUid() + ") not modified. not allowed for "
					+ token.getEmail());
			return c;
		}

		String q = "update Contact SET "
				+ "contact_firstname=?, "
				+ "contact_lastname=?, contact_origin=?, contact_userupdate=?, "
				+ "contact_aka=?, contact_title=?, contact_service=?, contact_company=?, contact_comment=?, "
				+ "contact_suffix=?, contact_manager=?, contact_middlename=?, contact_assistant=?, contact_spouse=? "
				+ "WHERE contact_id=?";
		logger.info("modify contact with id=" + c.getUid() + " entityId="
				+ c.getEntityId());

		Connection con = null;
		PreparedStatement ps = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);
			int idx = 1;
			ps.setString(idx++, c.getFirstname());
			ps.setString(idx++, c.getLastname());
			ps.setString(idx++, token.getOrigin());
			ps.setInt(idx++, token.getObmId());

			ps.setString(idx++, c.getAka());
			ps.setString(idx++, c.getTitle());
			ps.setString(idx++, c.getService());
			ps.setString(idx++, c.getCompany());
			ps.setString(idx++, c.getComment());

			ps.setString(idx++, c.getSuffix());
			ps.setString(idx++, c.getManager());
			ps.setString(idx++, c.getMiddlename());
			ps.setString(idx++, c.getAssistant());
			ps.setString(idx++, c.getSpouse());

			ps.setInt(idx++, c.getUid());
			ps.executeUpdate();

			createOrUpdateAddresses(con, c.getEntityId(), c.getAddresses());
			createOrUpdateEmails(con, c.getEntityId(), c.getEmails());
			createOrUpdatePhones(con, c.getEntityId(), c.getPhones());
			createOrUpdateWebsites(con, c.getEntityId(), c.getWebsites());
			createOrUpdateIMIdentifiers(con, c.getEntityId(), c
					.getImIdentifiers());
			ut.commit();
		} catch (Throwable se) {
			ObmHelper.rollback(ut);
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, null);
		}
		return c;
	}

	private boolean hasRightsOn(AccessToken token, int contactUid) {

		String q = "select contact_usercreate="
				+ token.getObmId()
				+ " or urights.entityright_write=1 or grights.entityright_write=1 or prights.entityright_write=1 "
				+

				"FROM Contact "
				+ "INNER JOIN ContactEntity ON contactentity_contact_id=contact_id "
				+ "LEFT JOIN EntityRight urights ON "
				+ "(urights.entityright_entity_id=contactentity_entity_id AND "
				+ "urights.entityright_consumer_id=(select userentity_entity_id FROM UserEntity WHERE userentity_user_id=?)) "
				+ "LEFT JOIN EntityRight grights ON grights.entityright_entity_id=contactentity_entity_id "
				+ "AND grights.entityright_consumer_id IN ("
				+ MY_GROUPS_QUERY
				+ ") "
				+ "LEFT JOIN EntityRight prights ON prights.entityright_entity_id=contactentity_entity_id AND prights.entityright_consumer_id IS NULL "
				+ "WHERE contact_id=?";

		boolean ret = false;

		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);
			int idx = 1;

			ps.setInt(idx++, token.getObmId());
			ps.setInt(idx++, token.getObmId());
			ps.setInt(idx++, contactUid);

			rs = ps.executeQuery();
			if (rs.next()) {
				ret = rs.getBoolean(1);
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		return ret;
	}

	/**
	 * @param id
	 * @return the contact with the given id if it is not archived
	 */
	public Contact findContact(AccessToken token, int id) {
		String q = "SELECT "
				+ CONTACT_SELECT_FIELDS
				+ ", now() as last_sync FROM Contact, ContactEntity WHERE "
				+ "contact_id=? AND contactentity_contact_id=contact_id AND contact_archive != 1";

		int idx = 1;
		Contact ret = null;

		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);
			ps.setInt(idx++, id);
			rs = ps.executeQuery();
			if (rs.next()) {
				ret = contactFromCursor(rs);
				Map<Integer, Contact> entityContact = new HashMap<Integer, Contact>();
				entityContact.put(ret.getEntityId(), ret);
				loadPhones(con, entityContact);
				loadIMIdentifiers(con, entityContact);
				loadWebsites(con, entityContact);
				loadAddresses(token, con, entityContact);
				loadEmails(con, entityContact);
				loadBirthday(con, entityContact);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		return ret;
	}

	/**
	 * bulk loads all emails of the given entities (contacts)
	 * 
	 * @param con
	 * @param entityContact
	 *            entityId / Contact map
	 */
	private void loadEmails(Connection con, Map<Integer, Contact> entityContact) {
		String q = "select email_entity_id, email_label, email_address FROM Email where email_entity_id IN ("
				+ buildIdList(entityContact.keySet()) + ")";
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery(q);
			while (rs.next()) {
				Contact c = entityContact.get(rs.getInt(1));
				Email p = new Email(rs.getString(3));
				c.addEmail(rs.getString(2), p);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	private void loadAddresses(AccessToken token, Connection con,
			Map<Integer, Contact> entityContact) {
		String q = "select address_entity_id, address_label, "
				+ "address_street, address_zipcode, address_expresspostal, address_town, address_country, address_state "
				+ "FROM Address where address_entity_id IN ("
				+ buildIdList(entityContact.keySet()) + ")";
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery(q);
			while (rs.next()) {
				Contact c = entityContact.get(rs.getInt(1));
				Address p = new Address(rs.getString(3), rs.getString(4), rs
						.getString(5), rs.getString(6), getCountryName(token,
						con, rs.getString(7)), rs.getString(8));
				c.addAddress(rs.getString(2), p);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	private void loadWebsites(Connection con,
			Map<Integer, Contact> entityContact) {
		String q = "select website_entity_id, website_label, website_url FROM Website where website_entity_id IN ("
				+ buildIdList(entityContact.keySet()) + ")";
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery(q);
			while (rs.next()) {
				Contact c = entityContact.get(rs.getInt(1));
				Website w = new Website(rs.getString(3));
				c.addWebsite(rs.getString(2), w);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	private void loadIMIdentifiers(Connection con,
			Map<Integer, Contact> entityContact) {
		String q = "select im_entity_id, im_label, im_address, im_protocol FROM IM where im_entity_id IN ("
				+ buildIdList(entityContact.keySet()) + ")";
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery(q);
			while (rs.next()) {
				Contact c = entityContact.get(rs.getInt(1));
				InstantMessagingId p = new InstantMessagingId(rs.getString(4),
						rs.getString(3));
				c.addIMIdentifier(rs.getString(2), p);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	/**
	 * Creates a comma separated list of id usable in IN (x,y,z) SQL queries
	 * 
	 * @param set
	 * @return
	 */
	private String buildIdList(Set<Integer> set) {
		StringBuilder sb = new StringBuilder(10 * set.size());
		sb.append("0");
		for (Integer i : set) {
			sb.append(",");
			sb.append(i);
		}
		return sb.toString();
	}

	private void loadPhones(Connection con, Map<Integer, Contact> entityContact) {
		String q = "select phone_entity_id, phone_label, phone_number FROM Phone where phone_entity_id IN ("
				+ buildIdList(entityContact.keySet()) + ")";
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery(q);
			while (rs.next()) {
				Contact c = entityContact.get(rs.getInt(1));
				Phone p = new Phone(rs.getString(3));
				c.addPhone(rs.getString(2), p);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	/**
	 * @param c
	 */
	private Contact removeContact(AccessToken at, Contact c) {
		Connection con = null;
		PreparedStatement ps = null;
		UserTransaction ut = ObmHelper.getUserTransaction();
		try {
			ut.begin();
			con = ObmHelper.getConnection();
			ps = con
					.prepareStatement("UPDATE Contact set contact_archive=1, contact_origin=? WHERE contact_id=?");
			ps.setString(1, at.getOrigin());
			ps.setInt(2, c.getUid());
			ps.executeUpdate();
			ut.commit();
		} catch (Throwable t) {
			ObmHelper.rollback(ut);
			logger.error(t.getMessage(), t);
		} finally {
			ObmHelper.cleanup(con, ps, null);
		}

		return c;
	}

	/**
	 * @param uid
	 */
	public Contact removeContact(AccessToken at, int uid) {
		Contact c = findContact(at, uid);
		if (c == null) {
			return null;
		}
		if (!hasRightsOn(at, uid)) {
			logger.info("contact " + uid + " removal not permitted for "
					+ at.getEmail());
			return c;
		}
		return removeContact(at, c);

	}

	public Set<Integer> findRemovalCandidates(Date d, AccessToken at) {
		Set<Integer> l = new HashSet<Integer>();
		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection con = null;

		String q = "SELECT deletedcontact_contact_id FROM DeletedContact WHERE deletedcontact_user_id="
				+ at.getObmId();
		if (d != null) {
			q += " AND deletedcontact_timestamp >= ? ";
		}
		q += " UNION SELECT contact_id FROM Contact WHERE contact_archive=1 AND contact_privacy="
				+ ContactPrivacy.PRIVATE.asInt()
				+ " AND contact_usercreate="
				+ at.getObmId();
		if (d != null) {
			q += " AND contact_timeupdate >= ?";
		}
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);

			if (d != null) {
				ps.setTimestamp(1, new Timestamp(d.getTime()));
				ps.setTimestamp(2, new Timestamp(d.getTime()));
			}
			rs = ps.executeQuery();
			while (rs.next()) {
				l.add(new Integer(rs.getInt(1)));
			}

			logger.info("Returning " + l.size() + " contact(s) deleted");
		} catch (SQLException e) {
			logger.error("Could not find deleted contacts in OBM", e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		return l;
	}

	/**
	 * Return id of contacts that look similar (used by funambol)
	 * 
	 * @param contact
	 * @return String[]
	 */
	public List<String> findContactTwinKeys(AccessToken at, Contact contact) {
		List<String> ret = new LinkedList<String>();

		List<Contact> contacts = searchSimilar(at, contact, 1);
		for (Contact c : contacts) {
			ret.add(c.getUid().toString());
		}
		return ret;
	}

	/**
	 * Return country iso3166 searched by country name
	 * 
	 * @param con
	 * @param countryName
	 * @return
	 */
	private String getCountryIso3166(Connection con, String countryName) {
		if (countryName == null) {
			return null;
		}

		String q = "select country_iso3166 from Country where lower(trim(country_name))=? OR lower(country_iso3166)=?";

		PreparedStatement ps = null;
		ResultSet rs = null;
		String result = null;
		try {
			ps = con.prepareStatement(q);
			ps.setString(1, countryName.toLowerCase());
			ps.setString(2, countryName.toLowerCase());
			rs = ps.executeQuery();
			if (rs.next()) {
				result = rs.getString(1);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, ps, rs);
		}
		return result;
	}

	/**
	 * Return country name searched by country iso3166
	 * 
	 * @param token
	 * @param con
	 * @param isoCode
	 * @return
	 */
	private String getCountryName(AccessToken token, Connection con,
			String isoCode) {
		if (isoCode == null) {
			return null;
		}
		Map<String, String> cache = token.getIsoCodeToNameCache();
		if (cache.containsKey(isoCode)) {
			return cache.get(isoCode);
		}

		String q = "select country_name from Country where lower(country_lang)="
				+ userLangSelect(token.getObmId())
				+ " AND (lower(trim(country_name))=? OR lower(country_iso3166)=?)";

		PreparedStatement ps = null;
		ResultSet rs = null;
		String result = null;
		try {
			ps = con.prepareStatement(q);
			ps.setString(1, isoCode.toLowerCase());
			ps.setString(2, isoCode.toLowerCase());
			rs = ps.executeQuery();
			if (rs.next()) {
				result = rs.getString(1);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, ps, rs);
		}
		cache.put(isoCode, result);
		return result;
	}

	private String userLangSelect(int obmId) {
		String q = "(SELECT lower(userobmpref_value) "
				+ "FROM UserObmPref "
				+ "WHERE  userobmpref_option='set_lang' AND (userobmpref_user_id = "
				+ obmId
				+ " OR ( "
				+ "userobmpref_user_id IS NULL AND "
				+ "userobmpref_option  NOT IN (SELECT userobmpref_option FROM UserObmPref WHERE userobmpref_user_id ="
				+ obmId + "))))";
		return q;
	}

	/**
	 * Search contacts that look "similar" to the given contact.
	 * 
	 * @param at
	 * @param query
	 * @param limit
	 * @return
	 */
	public List<Contact> searchSimilar(AccessToken at, Contact c, int limit) {
		String q = "SELECT "
				+ CONTACT_SELECT_FIELDS
				+ ", now() as last_sync FROM Contact "
				+ "INNER JOIN ContactEntity ON contactentity_contact_id=contact_id "
				+ "INNER JOIN SynchedContact ON (synchedcontact_contact_id=contact_id AND synchedcontact_user_id="
				+ at.getObmId() + ") ";

		if (c.getEmails().size() > 0) {
			q += "LEFT JOIN Email ON email_entity_id=contactentity_entity_id ";
		}

		if (c.getPhones().containsKey("CELL;VOICE;X-OBM-Ref1")) {
			q += "LEFT JOIN Phone ON (phone_entity_id=contactentity_entity_id AND phone_label='CELL;VOICE;X-OBM-Ref1') ";
		}

		q += "LEFT JOIN EntityRight urights ON "
				+ "(urights.entityright_entity_id=contactentity_entity_id AND "
				+ "urights.entityright_consumer_id=(select userentity_entity_id FROM UserEntity WHERE userentity_user_id=?)) "
				+ "LEFT JOIN EntityRight grights ON grights.entityright_entity_id=contactentity_entity_id "
				+ "AND grights.entityright_consumer_id IN ("
				+ MY_GROUPS_QUERY
				+ ") "
				+ "LEFT JOIN EntityRight prights ON prights.entityright_entity_id=contactentity_entity_id AND prights.entityright_consumer_id IS NULL "
				+ "WHERE "
				+ "((contact_privacy=? AND contact_archive != 1 AND contact_usercreate=?) OR "
				+ "(contact_privacy=? AND contact_archive != 1) OR "
				+ "(contact_archive != 1 AND (urights.entityright_read=1 OR grights.entityright_read=1 OR prights.entityright_read=1))) ";

		if (c.getFirstname() != null && c.getFirstname().length() > 0) {
			q += " AND lower(contact_firstname) = ? ";
		}
		if (c.getLastname() != null && c.getLastname().length() > 0) {
			q += " AND lower(contact_lastname) = ? ";
		}

		if (c.getEmails().size() > 0) {
			q += "AND (email_address IS NULL ";
			for (int i = 0; i < c.getEmails().size(); i++) {
				q += " OR lower(email_address) = ? ";
			}
			q += ") ";
		}
		if (c.getPhones().containsKey("CELL;VOICE;X-OBM-Ref1")) {
			q += "AND (phone_number IS NULL OR phone_number=?) ";
		}

		int idx = 1;
		List<Contact> found = new LinkedList<Contact>();
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);

			// userentity_user_id
			ps.setInt(idx++, at.getObmId());
			// my groups
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PRIVATE.asInt());
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PUBLIC.asInt());

			// values
			if (c.getFirstname() != null && c.getFirstname().length() > 0) {
				ps.setString(idx++, c.getFirstname().toLowerCase());
			}

			if (c.getLastname() != null && c.getLastname().length() > 0) {
				ps.setString(idx++, c.getLastname().toLowerCase());
			}

			for (String s : c.getEmails().keySet()) {
				ps.setString(idx++, c.getEmails().get(s).getEmail()
						.toLowerCase());
			}

			if (c.getPhones().containsKey("CELL;VOICE;X-OBM-Ref1")) {
				ps.setString(idx++, c.getPhones().get("CELL;VOICE;X-OBM-Ref1")
						.getNumber());
			}

			rs = ps.executeQuery();
			Map<Integer, Contact> entityContact = new HashMap<Integer, Contact>();

			while (rs.next()) {
				int entity = rs.getInt("contactentity_entity_id");
				if (!entityContact.containsKey(entity)) {
					Contact ct = contactFromCursor(rs);
					entityContact.put(ct.getEntityId(), ct);
					found.add(ct);
				}
			}
			rs.close();
			rs = null;

			if (!entityContact.isEmpty()) {
				loadPhones(con, entityContact);
				loadIMIdentifiers(con, entityContact);
				loadWebsites(con, entityContact);
				loadAddresses(at, con, entityContact);
				loadEmails(con, entityContact);
				loadBirthday(con, entityContact);
			}

		} catch (Throwable se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		logger.info("[" + at.getUser() + "] searchSimilar for '"
				+ c.getLastname() + "' returned " + found.size()
				+ " contact(s)");

		return found;
	}

	/**
	 * Search contacts. Query will match against lastname, firstname & email
	 * prefixes.
	 * 
	 * @param at
	 * @param query
	 * @param limit
	 * @return
	 */
	public List<Contact> searchContact(AccessToken at, String query, int limit) {

		Contact c = new Contact();
		c.setLastname(query + "%");
		c.setFirstname(query + "%");
		Email e = new Email(query + "%");
		c.addEmail("lbl", e);

		String q = "SELECT "
				+ CONTACT_SELECT_FIELDS
				+ ", now() as last_sync FROM Contact "
				+ "INNER JOIN ContactEntity ON contactentity_contact_id=contact_id "
				+ "INNER JOIN SynchedContact ON (synchedcontact_contact_id=contact_id AND synchedcontact_user_id="
				+ at.getObmId()
				+ ") "
				+ "LEFT JOIN Email ON email_entity_id=contactentity_entity_id "
				+ "LEFT JOIN EntityRight urights ON "
				+ "(urights.entityright_entity_id=contactentity_entity_id AND "
				+ "urights.entityright_consumer_id=(select userentity_entity_id FROM UserEntity WHERE userentity_user_id=?)) "
				+ "LEFT JOIN EntityRight grights ON grights.entityright_entity_id=contactentity_entity_id "
				+ "AND grights.entityright_consumer_id IN ("
				+ MY_GROUPS_QUERY
				+ ") "
				+ "LEFT JOIN EntityRight prights ON prights.entityright_entity_id=contactentity_entity_id AND prights.entityright_consumer_id IS NULL "
				+ "WHERE "
				+ "((contact_privacy=? AND contact_archive != 1 AND contact_usercreate=?) OR "
				+ "(contact_privacy=? AND contact_archive != 1) OR "
				+ "(contact_archive != 1 AND (urights.entityright_read=1 OR grights.entityright_read=1 OR prights.entityright_read=1)))";

		q += " AND (lower(contact_firstname) like ? OR lower(contact_lastname) like ? OR (FALSE";

		for (int i = 0; i < c.getEmails().size(); i++) {
			q += " OR lower(email_address) like ?";
		}

		q += "))";

		int idx = 1;
		List<Contact> found = new LinkedList<Contact>();
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(q);

			// userentity_user_id
			ps.setInt(idx++, at.getObmId());
			// my groups
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PRIVATE.asInt());
			ps.setInt(idx++, at.getObmId());

			ps.setInt(idx++, ContactPrivacy.PUBLIC.asInt());

			ps.setString(idx++, c.getFirstname().toLowerCase());
			ps.setString(idx++, c.getLastname().toLowerCase());
			for (String s : c.getEmails().keySet()) {
				ps.setString(idx++, c.getEmails().get(s).getEmail()
						.toLowerCase());
			}

			rs = ps.executeQuery();
			Map<Integer, Contact> entityContact = new HashMap<Integer, Contact>();

			int i = 0;
			while (rs.next() && i < limit) {
				int entity = rs.getInt("contactentity_entity_id");
				if (!entityContact.containsKey(entity)) {
					Contact ct = contactFromCursor(rs);
					entityContact.put(ct.getEntityId(), ct);
					found.add(ct);
					i++;
				}
			}
			rs.close();
			rs = null;

			if (!entityContact.isEmpty()) {
				loadPhones(con, entityContact);
				loadIMIdentifiers(con, entityContact);
				loadWebsites(con, entityContact);
				loadAddresses(at, con, entityContact);
				loadEmails(con, entityContact);
				loadBirthday(con, entityContact);
			}

		} catch (Throwable se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		logger.info("[" + at.getUser() + "] search for '" + c.getLastname()
				+ "' returned " + found.size() + " contact(s)");

		return found;
	}

}
