package fr.aliacom.obm.common.setting;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;

import fr.aliacom.obm.utils.ObmHelper;

public class SettingHome {
	
	private Log logger = LogFactory.getLog(getClass());
	
	private static SettingHome instance = new SettingHome();

	/**
	 * @return the home instance
	 */
	public static SettingHome getInstance() {
		return instance;
	}

	public Map<String, String> getSettings(AccessToken at) {
		Map<String, String> data = new HashMap<String, String>();
		
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		String domain = at.getDomain();
		String user = at.getUser();
		
		try {
			con = ObmHelper.getConnection();
			
			String q = "select userobmpref_option, userobmpref_value from UserObmPref "
					+ "where userobmpref_user_id is null "
					+ "and userobmpref_option not in "
					+ "(select userobmpref_option from UserObmPref where userobmpref_user_id=(select userobm_id from UserObm "
					+ "where userobm_login=? ";
			if (domain != null) {
				q += "and userobm_domain_id=(select domain_id from Domain where domain_name=?) ";
			}
			q += ")) "
					+ "union "
					+ "select userobmpref_option, userobmpref_value from UserObmPref "
					+ "where userobmpref_user_id=(select userobm_id from UserObm "
					+ "where userobm_login=? ";
			if (domain != null) {
				q += "and userobm_domain_id=(select domain_id from Domain where domain_name=?) ";
			}
			q += " )";
			ps = con.prepareStatement(q);
			int idx = 1;
			ps.setString(idx++, user);
			if (domain != null) {
				ps.setString(idx++, domain);
			}
			logger.info("login: "+user);
			ps.setString(idx++, user);
			if (domain != null) {
				ps.setString(idx++, domain);
			}
			rs = ps.executeQuery();
			while (rs.next()) {
				logger.info("["+user+"] adding pref "+rs.getString(1)+" : "+rs.getString(2));
				data.put(rs.getString(1), rs.getString(2));
			}
		} catch (SQLException se) {
			logger.error("Error fetching preferences for " + user + " at "
					+ domain, se);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}

		logger.info("return " + data.size() + " settings for userid: " + user
				+ " at " + domain);
		return data;
	}
	
	public String getUserDomain(String login) {
		Connection con = null;
		ResultSet rs = null;
		PreparedStatement ps = null;
		String userDomain = null;
		try {
			con = ObmHelper.getConnection();

			String query;
			query = "SELECT domain_name" + " FROM UserObm, Domain WHERE"
					+ " userobm_domain_id = domain_id AND userobm_login = ?";

			ps = con.prepareStatement(query);

			ps.setString(1, login);
			rs = ps.executeQuery();

			if (rs.next()) {
				do {
					userDomain = rs.getString(1);
				} while (rs.next());
			}

		} catch (SQLException e) {
			logger.error("Could not find user domain", e);
		} finally {
			try {
				ObmHelper.cleanup(con, ps, rs);
			} catch (Exception e) {
				logger.error("Could not clean up jdbc stuff");
			}
		}
		return userDomain;
	}

}
