/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.freebusy;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fr.aliacom.obm.common.ObmDomain;
import fr.aliacom.obm.common.ObmUser;
import fr.aliacom.obm.common.UserManagement;
import fr.aliacom.obm.common.calendar.CalendarService;
import fr.aliacom.obm.utils.Helper;

public class FreeBusyProvider extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3887606350629311688L;
	private Log logger = LogFactory.getLog(FreeBusyProvider.class);

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {

		String reqString = request.getRequestURI();
		logger.info("FreeBusyProvider : reqString: '" + reqString + "'");

		// get user from reqString : format
		// "/obm-sync/freebusy/firstname.lastname@obmdomain"

		String email = reqString.substring(reqString.lastIndexOf("/") + 1);
		logger.info("freebusy email : '" + email + "'");

		String[] splited = email.split("@", 2);
		String username = splited[0];
		String userDomain = null;
		if (splited.length == 2) {
			userDomain = splited[1];
		}

		ObmDomain domain = UserManagement.getInstance().findDomainByName(
				userDomain);
		ObmUser user = UserManagement.getInstance().findUser(email, email,
				domain);

		if (user == null) {
			logger.info("FreeBusyProvider : user not found : '" + email + "'");
			return;
		}

		File tmp = null;
		try {
			tmp = File.createTempFile(username, ".vfb");
			RandomAccessFile raf = new RandomAccessFile(tmp, "rw");

			writeBeginVFB(raf);
			writeContentVBF(raf, email, user, domain);
			writeEndVFB(raf);

			raf.close();

			SimpleDateFormat formatter = new SimpleDateFormat(
					"EEE, dd MMM yyyy HH:mm:ss z");
			response.setHeader("Expires", formatter.format(new Date()));

			response.setContentType("text/plain");
			response.setContentLength((int) tmp.length());

			transfer(new FileInputStream(tmp), response.getOutputStream());
		} finally {
			tmp.delete();
		}
	}

	private void transfer(FileInputStream in, ServletOutputStream out)
			throws IOException {

		byte[] buffer = new byte[128];
		while (true) {
			int read = in.read(buffer);
			if (read == -1) {
				break;
			}
			out.write(buffer, 0, read);
		}

	}

	private void writeContentVBF(RandomAccessFile raf, String email,
			ObmUser user, ObmDomain domain) throws IOException {
		raf.writeBytes("ORGANIZER:" + email + "\r\n");

		java.util.Calendar dnow = Calendar.getInstance(TimeZone
				.getTimeZone("GMT"));
		Date dstamp = dnow.getTime();
		dnow.add(java.util.Calendar.MONTH, -1);
		Date dstart = dnow.getTime();
		dnow.add(java.util.Calendar.MONTH, 2);
		Date dend = dnow.getTime();

		raf.writeBytes("DTSTAMP:" + Helper.getUTCFormat(dstamp) + "\r\n");
		raf.writeBytes("DTSTART:" + Helper.getUTCFormat(dstart) + "\r\n");
		raf.writeBytes("DTEND:" + Helper.getUTCFormat(dend) + "\r\n");

		String[] freebusy = CalendarService.getInstance().getFreeBusy(
				user.getLogin(), dstart, dend, domain);
		for (int i = 0; i < freebusy.length; i++) {
			raf.writeBytes(freebusy[i]);
		}
	}

	private void writeBeginVFB(RandomAccessFile raf) throws IOException {
		raf.writeBytes("BEGIN:VCALENDAR\r\n");
		raf.writeBytes("PRODID:-//Aliasource//OBM V 2.x//EN\r\n");
		raf.writeBytes("VERSION:2.0\r\n");
		raf.writeBytes("METHOD:PUBLISH\r\n");
		raf.writeBytes("BEGIN:VFREEBUSY\r\n");
	}

	private void writeEndVFB(RandomAccessFile raf) throws IOException {
		raf.writeBytes("END:VFREEBUSY\r\n");
		raf.writeBytes("END:VCALENDAR\r\n");
		raf.writeBytes("\r\n");
	}
}