/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.ldap;

import java.io.InputStream;

import org.obm.sync.utils.DOMUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import fr.aliacom.obm.services.constant.ConstantService;

/**
 * Contient la liste des ldaps sur lesquels on peut essayer de s'authentifier
 * 
 * @author tom
 * 
 */
class LDAPAuthConfig {

	private LDAPDirectory dir;

	LDAPAuthConfig(InputStream conf) {
		this.dir = parseConf(conf);
	}

	 LDAPAuthConfig() {
		 ConstantService cs = ConstantService.getInstance();
		 String uri = cs.getStringValue("auth-ldap-server");
		 String baseDN = cs.getStringValue("auth-ldap-basedn").replace("\"", "");
		 String userFilter = cs.getStringValue("auth-ldap-filter").replace("\"", "");
		 dir = new LDAPDirectory(uri, userFilter, null, null, baseDN, null, null);
	 }

	/**
	 * Voir ldap-auth.xml
	 * 
	 * @param conf
	 * @return
	 */
	private LDAPDirectory parseConf(InputStream conf) {
		LDAPDirectory ldapDir = null;
		try {
			Document dom = DOMUtils.parse(conf);
			Element root = dom.getDocumentElement();
			Element dir = DOMUtils.getUniqueElement(root, "directory");
			String uri = DOMUtils.getElementText(dir, "uri");
			String userPattern = DOMUtils.getElementText(dir, "userPattern");
			String rootDN = DOMUtils.getElementText(dir, "rootDN");
			String rootPW = DOMUtils.getElementText(dir, "rootPW");
			String baseDN = DOMUtils.getElementText(dir, "baseDN");
			String matchField = DOMUtils.getElementText(dir, "matchField");
			String obmDomain = DOMUtils.getElementText(dir, "obmDomain");

			ldapDir = new LDAPDirectory(uri, userPattern, rootDN, rootPW,
					baseDN, matchField, obmDomain);
		} catch (Exception e) {
			throw new BrokenLDAPAuthConfiguration(e);
		}
		return ldapDir;
	}

	public LDAPDirectory getDirectory() {
		return dir;
	}

}
