/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.utils;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.ParticipationState;

public class Helper {

	private static final String HEX_DIGITS = "0123456789abcdef";
	private static final String DATE_UTC_PATTERN = "yyyyMMdd'T'HHmmss'Z'";
	private static final SimpleDateFormat dateFormatUTC = new SimpleDateFormat(
			DATE_UTC_PATTERN);

	private static Log logger;
	static {
		logger = LogFactory.getLog(ObmHelper.class);
		dateFormatUTC.setTimeZone(TimeZone.getTimeZone("UTC"));
	}

	public static String constructEmailFromList(String listofmail,
			String domain) {

		String[] lemail = null;
		if (listofmail != null) {
			lemail = listofmail.split("\r\n");
			if (lemail.length > 0) {

				if (lemail[0].contains("@")) {
					return lemail[0];
				} else {
					return lemail[0] + "@" + domain;
				}
			} else {
				return "";
			}
		} else {
			return "";
		}
	}

	public static String getMD5Diggest(String plaintext) {
		MessageDigest mg = null;
		try {
			mg = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			logger.error(e.getMessage(), e);
		}
		mg.update(plaintext.getBytes());
		return toHexString(mg.digest());
	}

	private static String toHexString(byte[] param) {
		StringBuffer sb = new StringBuffer(param.length * 2);
		for (int i = 0; i < param.length; i++) {
			int b = param[i] & 0xFF;
			sb.append(HEX_DIGITS.charAt(b >>> 4)).append(
					HEX_DIGITS.charAt(b & 0xF));
		}
		return sb.toString();
	}

	/**
	 * Returns the given date in utc format.
	 * 
	 * @param date
	 *            Date
	 * @return String
	 */
	public static String getUTCFormat(Date date) {
		String utc = null;
		if (date != null) {
			utc = dateFormatUTC.format(date);
		}
		return utc;
	}

	/**
	 * Return true if user has write right on calendar
	 * 
	 * @param rights
	 * @param login
	 * @param calendar
	 * @return
	 */
	public static boolean hasWriteRightOn(List<CalendarInfo> rights,
			String ownerLogin) {
		if (logger.isInfoEnabled()) {
			logger.info("Rights list for ownerLogin " + ownerLogin
					+ " contains:");
			for (CalendarInfo ci : rights) {
				logger.info("     ci.uid: " + ci.getUid() + " ci.read: "
						+ ci.isRead() + " ci.write: " + ci.isWrite());
			}
		}
		CalendarInfo searchRW = new CalendarInfo();
		searchRW.setRead(true);
		searchRW.setUid(ownerLogin);
		searchRW.setWrite(true);
		
		int searchIndex = rights.indexOf(searchRW);
		if (searchIndex != -1) {
			return rights.get(searchIndex).isWrite();
		} else {
			return false;
		}
	}

	/**
	 * Reset status of event
	 * 
	 * @param event
	 */
	public static void resetAttendeesStatus(Event event) {
		if (event.getAttendees() != null) {
			for (Attendee att : event.getAttendees()) {
				att.setState(ParticipationState.NEEDSACTION);
			}
		}
	}
}
