package fr.aliacom.obm.utils;

import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.fortuna.ical4j.data.CalendarBuilder;
import net.fortuna.ical4j.data.ParserException;
import net.fortuna.ical4j.model.Calendar;
import net.fortuna.ical4j.model.Component;
import net.fortuna.ical4j.model.ComponentList;
import net.fortuna.ical4j.model.DateList;
import net.fortuna.ical4j.model.DateTime;
import net.fortuna.ical4j.model.Dur;
import net.fortuna.ical4j.model.Parameter;
import net.fortuna.ical4j.model.Property;
import net.fortuna.ical4j.model.PropertyList;
import net.fortuna.ical4j.model.Recur;
import net.fortuna.ical4j.model.WeekDay;
import net.fortuna.ical4j.model.WeekDayList;
import net.fortuna.ical4j.model.component.CalendarComponent;
import net.fortuna.ical4j.model.component.VAlarm;
import net.fortuna.ical4j.model.component.VEvent;
import net.fortuna.ical4j.model.component.VToDo;
import net.fortuna.ical4j.model.parameter.Cn;
import net.fortuna.ical4j.model.parameter.CuType;
import net.fortuna.ical4j.model.parameter.PartStat;
import net.fortuna.ical4j.model.parameter.Role;
import net.fortuna.ical4j.model.parameter.Rsvp;
import net.fortuna.ical4j.model.parameter.Value;
import net.fortuna.ical4j.model.property.CalScale;
import net.fortuna.ical4j.model.property.Categories;
import net.fortuna.ical4j.model.property.Clazz;
import net.fortuna.ical4j.model.property.DateProperty;
import net.fortuna.ical4j.model.property.Description;
import net.fortuna.ical4j.model.property.DtEnd;
import net.fortuna.ical4j.model.property.DtStart;
import net.fortuna.ical4j.model.property.Due;
import net.fortuna.ical4j.model.property.ExDate;
import net.fortuna.ical4j.model.property.Location;
import net.fortuna.ical4j.model.property.Organizer;
import net.fortuna.ical4j.model.property.PercentComplete;
import net.fortuna.ical4j.model.property.Priority;
import net.fortuna.ical4j.model.property.ProdId;
import net.fortuna.ical4j.model.property.RRule;
import net.fortuna.ical4j.model.property.RecurrenceId;
import net.fortuna.ical4j.model.property.Status;
import net.fortuna.ical4j.model.property.Summary;
import net.fortuna.ical4j.model.property.Transp;
import net.fortuna.ical4j.model.property.Trigger;
import net.fortuna.ical4j.model.property.Uid;
import net.fortuna.ical4j.model.property.Version;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventOpacity;
import org.obm.sync.calendar.EventRecurrence;
import org.obm.sync.calendar.EventType;
import org.obm.sync.calendar.ParticipationRole;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.calendar.RecurrenceKind;

public class Ical4jHelper {

	private static Log logger = LogFactory.getLog(Ical4jHelper.class);

	public static List<Event> parseICS(InputStream icsStream, AccessToken token)
			throws IOException, ParserException {
		List<Event> ret = new LinkedList<Event>();
		CalendarBuilder builder = new CalendarBuilder();
		Calendar calendar = builder.build(icsStream);

		if (calendar != null) {
			ComponentList comps = getComponents(calendar, Component.VEVENT);
			Map<String, Event> mapEvents = getEvents(comps, Component.VEVENT,
					token);
			ret.addAll(mapEvents.values());

			comps = getComponents(calendar, Component.VTODO);
			Map<String, Event> mapTodo = getEvents(comps, Component.VTODO,
					token);
			ret.addAll(mapTodo.values());
		}

		return ret;
	}

	@SuppressWarnings("unchecked")
	private static Map<String, Event> getEvents(ComponentList cl,
			String typeCalendar, AccessToken token) {
		Map<String, Event> mapEvents = new HashMap<String, Event>();
		for (Iterator<Component> it = cl.iterator(); it.hasNext();) {
			Component comp = it.next();
			if (comp != null) {

				if (Component.VEVENT.equals(typeCalendar)) {
					VEvent vEvent = (VEvent) comp;
					if (vEvent.getRecurrenceId() == null) {
						Event event = getEvent(vEvent, token);
						mapEvents.put(event.getExtId(), event);
					} else {
						Event eexcep = getEvent(vEvent, token);
						Event event = mapEvents.get(eexcep.getExtId());

						if (event != null) {
							eexcep.setExtId(null);
							event.getRecurrence().addEventException(eexcep);
							event.getRecurrence().addException(
									getRecurrenceId(vEvent));
						}
					}
				} else if (Component.VTODO.equals(typeCalendar)) {
					VToDo vTodo = (VToDo) comp;
					if (vTodo.getRecurrenceId() == null) {
						Event event = getEvent(vTodo, token);
						mapEvents.put(event.getExtId(), event);
					} else {
						Event eexcep = getEvent(vTodo, token);
						Event event = mapEvents.get(eexcep.getExtId());

						if (event != null) {
							eexcep.setExtId(null);
							event.getRecurrence().addEventException(eexcep);
							event.getRecurrence().addException(
									getRecurrenceId(vTodo));
						}
					}
				}
			}
		}
		return mapEvents;
	}

	private static Date getRecurrenceId(VEvent vEvent) {
		return vEvent.getRecurrenceId().getDate();
	}

	private static Date getRecurrenceId(VToDo vTodo) {
		return vTodo.getRecurrenceId().getDate();
	}

	public static Event getEvent(VEvent vEvent, AccessToken token) {
		Event event = new Event();
		event.setType(EventType.VEVENT);
		appendSummary(event, vEvent.getSummary());
		appendDescription(event, vEvent.getDescription());
		appendUid(event, vEvent.getUid());
		appendPrivacy(event, vEvent.getClassification());
		appendOwner(event, vEvent.getOrganizer());
		appendCategory(event, vEvent.getProperty(Property.CATEGORIES));
		appendLocation(event, vEvent.getLocation());
		appendDate(event, vEvent.getStartDate());
		appendDuration(event, vEvent.getStartDate(), vEvent.getEndDate());
		appendAllDay(event, vEvent.getStartDate(), vEvent.getEndDate());
		appendPriority(event, vEvent.getPriority());

		appendAttendees(event, vEvent, token);
		appendRecurence(event, vEvent);
		appendAlert(event, vEvent.getAlarms(), vEvent.getProperties());
		event.setOpacity(EventOpacity.TRANSPARENT);
		
		Property propXMoz = vEvent.getProperty("X-MOZ-LASTACK");
		if(propXMoz != null){
			if(event.getRecurrence() != null && !RecurrenceKind.none.equals(event.getRecurrence().getKind())){
				Event eventRecur = event.clone();
				eventRecur.setDatabaseId(0);
				eventRecur.setEntityId(null);
				eventRecur.setExtId(null);
				eventRecur.setRecurrence(null);
				eventRecur.setUid(null);
				eventRecur.setParentId(event.getDatabaseId());
				event.getRecurrence().addEventException(eventRecur);
				event.getRecurrence().addException(eventRecur.getDate());
			}
		}
		
		return event;
	}

	public static Event getEvent(VToDo vTodo, AccessToken token) {
		Event event = new Event();
		event.setType(EventType.VTODO);
		appendSummary(event, vTodo.getSummary());
		appendDescription(event, vTodo.getDescription());
		appendUid(event, vTodo.getUid());
		appendPrivacy(event, vTodo.getClassification());
		appendOwner(event, vTodo.getOrganizer());
		appendCategory(event, vTodo.getProperty(Property.CATEGORIES));
		appendLocation(event, vTodo.getLocation());
		appendDate(event, vTodo.getStartDate());
		appendDuration(event, vTodo.getStartDate(), vTodo.getDue());
		appendAllDay(event, vTodo.getStartDate(), vTodo.getDue());
		appendPriority(event, vTodo.getPriority());

		appendAttendees(event, vTodo, token);
		appendRecurence(event, vTodo);
		appendAlert(event, vTodo.getAlarms(), vTodo.getProperties());
		appendPercent(event, vTodo.getPercentComplete(), token.getEmail());
		appendStatus(event, vTodo.getStatus(), token.getEmail());
		event.setOpacity(EventOpacity.TRANSPARENT);
		
		Property propXMoz = vTodo.getProperty("X-MOZ-LASTACK");
		if(propXMoz != null){
			if(event.getRecurrence() != null && !RecurrenceKind.none.equals(event.getRecurrence().getKind())){
				Event eventRecur = event.clone();
				eventRecur.setDatabaseId(0);
				eventRecur.setEntityId(null);
				eventRecur.setExtId(null);
				eventRecur.setRecurrence(null);
				eventRecur.setUid(null);
				eventRecur.setParentId(event.getDatabaseId());
				event.getRecurrence().addEventException(eventRecur);
				event.getRecurrence().addException(eventRecur.getDate());
			}
		}
		
		return event;
	}

	private static void appendStatus(Event event, Status status, String email) {
		if (status != null) {
			for (Attendee att : event.getAttendees()) {
				if (att.getEmail().equals(email)) {

					if (Status.VTODO_NEEDS_ACTION.equals(status)) {
						att.setState(ParticipationState.NEEDSACTION);
					} else if (Status.VTODO_IN_PROCESS.equals(status)) {
						att.setState(ParticipationState.INPROGRESS);
					} else if (Status.VTODO_COMPLETED.equals(status)) {
						att.setState(ParticipationState.COMPLETED);
					} else if (Status.VTODO_CANCELLED.equals(status)) {
						att.setState(ParticipationState.DECLINED);
					} else {
						att.setState(null);
					}
				}
			}
		}
	}

	private static void appendPercent(Event event,
			PercentComplete percentComplete, String email) {
		if (percentComplete != null) {
			for (Attendee att : event.getAttendees()) {
				if (att.getEmail().equals(email)) {
					att.setPercent(percentComplete.getPercentage());
				}
			}
		}
	}

	private static void appendPriority(Event event, Priority priority) {
		if (priority != null) {
			event.setPriority(new Integer(priority.getLevel()));
		}
	}

	private static void appendAllDay(Event event, DtStart startDate,
			DateProperty endDate) {
		if (endDate == null) {
			event.setAllday(true);
		} else {
			if (getDuration(startDate, endDate) == 86400) {
				event.setAllday(true);
			} else {
				event.setAllday(false);
			}
		}

	}

	private static void appendDuration(Event event, DtStart startDate,
			DateProperty due) {
		int duration = getDuration(startDate, due);
		event.setDuration(duration);
	}

	private static void appendDate(Event event, DtStart startDate) {
		if (startDate != null) {
			event.setDate(startDate.getDate());
		}

	}

	private static void appendLocation(Event event, Location location) {
		if (location != null) {
			event.setLocation(location.getValue());
		}
	}

	private static void appendCategory(Event event, Property category) {
		if (category != null) {
			event.setCategory(category.getValue());
		}
	}

	private static void appendOwner(Event event, Organizer organizer) {
		if (organizer != null) {
			Parameter cn = organizer.getParameter(Parameter.CN);
			String cnOrganizer = "";
			if (cn != null) {
				cnOrganizer = cn.getValue();
			}

			if (cnOrganizer != null && !"".equals(cnOrganizer)) {
				event.setOwner(cnOrganizer);
			} else {
				int mailToIndex = organizer.getValue().toLowerCase().indexOf(
						"mailto:");
				if (mailToIndex != -1) {
					event.setOwner(organizer.getValue().substring(
							mailToIndex + "mailto:".length()));
				}
			}

		}
	}

	private static void appendPrivacy(Event event, Clazz classification) {
		if (classification != null && Clazz.PRIVATE.equals(classification)) {
			event.setPrivacy(1);
		} else {
			event.setPrivacy(0);
		}

	}

	private static void appendUid(Event event, Uid uid) {
		if (uid != null) {
			event.setExtId(uid.getValue());
		}
	}

	private static void appendDescription(Event event, Description description) {
		if (description != null) {
			event.setDescription(description.getValue());
		}
	}

	private static void appendSummary(Event event, Summary summary) {
		if (summary != null) {
			event.setTitle(summary.getValue());
		}

	}

	public static String parseEvents(List<Event> listEvent) {

		Calendar calendar = initCalandar();

		for (Event event : listEvent) {
			VEvent vEvent = getVEvent(event);
			calendar.getComponents().add(vEvent);
		}
		return calendar.toString();
	}

	public static String parseEvent(Event event, AccessToken token) {
		Calendar calendar = initCalandar();
		if (EventType.VEVENT.equals(event.getType())) {
			VEvent vEvent = getVEvent(event);
			calendar.getComponents().add(vEvent);
			if (event.getRecurrence() != null) {
				for (Event ee : event.getRecurrence().getEventExceptions()) {
					VEvent eventExt = getVEvent(ee, event.getExtId());
					calendar.getComponents().add(eventExt);
				}
			}
		} else if (EventType.VTODO.equals(event.getType())) {
			VToDo vTodo = getVToDo(event, token);
			calendar.getComponents().add(vTodo);
			if (event.getRecurrence() != null) {
				for (Event ee : event.getRecurrence().getEventExceptions()) {
					VToDo todoExt = getVTodo(ee, event.getExtId(), token);
					calendar.getComponents().add(todoExt);
				}
			}
		}
		return calendar.toString();
	}

	public static VEvent getVEvent(Event event) {
		return getVEvent(event, null);
	}

	public static VEvent getVEvent(Event event, String parentExtID) {
		VEvent vEvent = new VEvent();
		PropertyList prop = vEvent.getProperties();

		appendUidToICS(prop, event, parentExtID);
		appendAttendeeToICS(prop, event);
		appendCategoryToICS(prop, event);
		appendDtStartToICS(prop, event);
		appendDtEndToICS(prop, event);
		appendDescriptionToICS(prop, event);
		appendLocationToICS(prop, event);
		appendTranspToICS(prop, event);
		appendOrganizerToICS(prop, event);
		appendPriorityToICS(prop, event);
		appendPrivacyToICS(prop, event);
		appendSummaryToICS(prop, event);
		appendRRuleToICS(prop, event);
		appendExDateToICS(prop, event);
		appendVAlarmToICS(vEvent.getAlarms(), event);
		appendRecurenceIdToICS(prop, event, parentExtID);
		
		return vEvent;
	}

	public static VToDo getVToDo(Event event, AccessToken token) {
		return getVTodo(event, null, token);
	}

	private static VToDo getVTodo(Event event, String parentExtID,
			AccessToken token) {
		VToDo vTodo = new VToDo();
		PropertyList prop = vTodo.getProperties();

		appendUidToICS(prop, event, parentExtID);
		appendAttendeeToICS(prop, event);
		appendCategoryToICS(prop, event);
		appendDtStartToICS(prop, event);
		appendDuedToICS(prop, event);
		appendDescriptionToICS(prop, event);
		appendLocationToICS(prop, event);
		appendTranspToICS(prop, event);
		appendOrganizerToICS(prop, event);
		appendPriorityToICS(prop, event);
		appendPrivacyToICS(prop, event);
		appendSummaryToICS(prop, event);
		appendRRuleToICS(prop, event);
		appendExDateToICS(prop, event);
		appendVAlarmToICS(vTodo.getAlarms(), event);
		appendRecurenceIdToICS(prop, event, parentExtID);
		appendPercentCompleteToICS(prop, event, token);
		appendStatusToICS(prop, event, token);

		return vTodo;
	}

	private static void appendStatusToICS(PropertyList prop, Event event,
			AccessToken token) {
		for (Attendee att : event.getAttendees()) {
			if (att.getEmail().equals(token.getEmail())) {
				if (ParticipationState.NEEDSACTION.equals(att.getState())) {
					prop.add(Status.VTODO_NEEDS_ACTION);
				} else if (ParticipationState.INPROGRESS.equals(att.getState())) {
					prop.add(Status.VTODO_IN_PROCESS);
				} else if (ParticipationState.COMPLETED.equals(att.getState())) {
					prop.add(Status.VTODO_COMPLETED);
				} else if (ParticipationState.DECLINED.equals(att.getState())) {
					prop.add(Status.VTODO_CANCELLED);
				} else {
					prop.add(new Status(""));
				}

			}
		}
	}

	private static void appendPercentCompleteToICS(PropertyList prop,
			Event event, AccessToken token) {
		for (Attendee att : event.getAttendees()) {
			if (att.getEmail().equals(token.getEmail())) {
				prop.add(new PercentComplete(att.getPercent()));
			}
		}

	}

	private static void appendDuedToICS(PropertyList prop, Event event) {
		if (event.getDuration() != 0) {
			Due dtEnd = getDue(event.getDate(), event.getDuration());
			if (dtEnd != null) {
				prop.add(dtEnd);
			}
		}
	}

	private static void appendRecurenceIdToICS(PropertyList prop, Event event,
			String parentExtID) {
		if (parentExtID != null && !"".equals(parentExtID)
				&& event.getParentId() > 0) {
			prop.add(getRecurrenceId(event));
		}
	}

	private static void appendVAlarmToICS(ComponentList prop, Event event) {
		VAlarm vAlarm = getVAlarm(event.getAlert());
		if (vAlarm != null) {
			prop.add(vAlarm);
		}
	}

	private static void appendExDateToICS(PropertyList prop, Event event) {
		Set<ExDate> exdates = getExDate(event);
		for (ExDate exdate : exdates) {
			prop.add(exdate);
		}
	}

	private static void appendRRuleToICS(PropertyList prop, Event event) {
		if (event.getParentId() == 0) {
			RRule rrule = getRRule(event);
			if (rrule != null) {
				prop.add(rrule);
			}
		}
	}

	private static void appendSummaryToICS(PropertyList prop, Event event) {
		prop.add(new Summary(event.getTitle()));
	}

	private static void appendPrivacyToICS(PropertyList prop, Event event) {
		prop.add(getClazz(event.getPrivacy()));
	}

	private static void appendPriorityToICS(PropertyList prop, Event event) {
		if (event.getPriority() != null) {
			prop.add(new Priority(event.getPriority()));
		}
	}

	private static void appendOrganizerToICS(PropertyList prop, Event event) {
		prop.add(getOrganizer(event));
	}

	private static void appendTranspToICS(PropertyList prop, Event event) {
		prop.add(getTransp(event.getOpacity()));
	}

	private static void appendLocationToICS(PropertyList prop, Event event) {
		if (event.getLocation() != null) {
			prop.add(new Location(event.getLocation()));
		}
	}

	private static void appendDescriptionToICS(PropertyList prop, Event event) {
		if (!isEmpty(event.getDescription())) {
			prop.add(new Description(event.getDescription()));
		}

	}

	private static void appendDtEndToICS(PropertyList prop, Event event) {
		DtEnd dtEnd = getDtEnd(event.getDate(), event.getDuration(), event
				.isAllday());
		if (dtEnd != null) {
			prop.add(dtEnd);
		}
	}

	private static void appendDtStartToICS(PropertyList prop, Event event) {
		prop.add(getDtStart(event));
	}

	private static void appendCategoryToICS(PropertyList prop, Event event) {
		if (!isEmpty(event.getCategory())) {
			prop.add(new Categories(event.getCategory()));
		}
	}

	private static void appendAttendeeToICS(PropertyList prop, Event event) {
		for (Attendee attendee : event.getAttendees()) {
			net.fortuna.ical4j.model.property.Attendee att = new net.fortuna.ical4j.model.property.Attendee();

			att.getParameters().add(CuType.INDIVIDUAL);

			att.getParameters().add(getPartStat(attendee));

			att.getParameters().add(Rsvp.TRUE);

			att.getParameters().add(getCn(attendee));

			att.getParameters().add(getRole(attendee));

			try {
				att.setValue("mailto:" + attendee.getEmail());
			} catch (URISyntaxException e) {
				logger.error(e.getMessage(), e);
			}
			prop.add(att);
		}

	}

	private static void appendUidToICS(PropertyList prop, Event event,
			String parentExtId) {
		if (event.getParentId() > 0 && !isEmpty(parentExtId)) {
			prop.add(new Uid(parentExtId));
		} else if (!isEmpty(event.getExtId())) {
			prop.add(new Uid(event.getExtId()));
		} else {
			prop.add(new Uid(event.getUid()));
		}

	}

	public static boolean isInIntervalDate(Event event, Date start, Date end) {
		Recur recur = Ical4jHelper.getRecur(event);
		if (recur != null) {
			DateList dl = recur.getDates(new DateTime(event.getDate()),
					new DateTime(end), Value.DATE_TIME);
			if (dl.size() > 0) {
				for (Object ob : dl) {
					DateTime date = (DateTime) ob;
					if (date.after(start) && date.before(end)) {
						return true;
					}
				}
			}
		} else {
			return true;
		}
		return false;
	}

	public static Recur getRecur(Event event) {
		Recur recur = null;
		EventRecurrence eventRecurrence = event.getRecurrence();
		if (eventRecurrence != null) {
			// String frequency = getFrequency(eventRecurrence);
			boolean isMonthyByDay = false;
			String frequency = "";
			RecurrenceKind kindRecur = eventRecurrence.getKind();
			if (RecurrenceKind.daily.equals(kindRecur)) {
				frequency = Recur.DAILY;
			} else if (RecurrenceKind.weekly.equals(kindRecur)) {
				frequency = Recur.WEEKLY;
			} else if (RecurrenceKind.monthlybydate.equals(kindRecur)) {
				frequency = Recur.MONTHLY;
			} else if (RecurrenceKind.monthlybyday.equals(kindRecur)) {
				frequency = Recur.MONTHLY;
				isMonthyByDay = true;

			} else if (RecurrenceKind.yearly.equals(kindRecur)) {
				frequency = Recur.YEARLY;
			} else if (RecurrenceKind.none.equals(kindRecur)) {
				frequency = "";
			}

			if (!"".equals(frequency)) {
				if (eventRecurrence.getEnd() == null) {
					recur = new Recur(frequency, null);
				} else {
					recur = new Recur(frequency, new DateTime(eventRecurrence
							.getEnd()));
				}

				if (isMonthyByDay) {
					GregorianCalendar cal = new GregorianCalendar();
					cal.setTime(event.getDate());
					recur.getDayList().add(WeekDay.getMonthlyOffset(cal));
				}

				recur.setInterval(eventRecurrence.getFrequence());

				Set<WeekDay> listDay = getListDay(eventRecurrence);

				for (WeekDay wd : listDay) {
					recur.getDayList().add(wd);
				}
			}
		}
		return recur;
	}

	public static Set<WeekDay> getListDay(EventRecurrence eventRecurrence) {
		Set<WeekDay> listDay = new HashSet<WeekDay>();
		String days = eventRecurrence.getDays();
		if (days != null && !"".equals(days)) {
			char[] c = days.toCharArray();
			List<WeekDay> listAcceptDay = new ArrayList<WeekDay>();
			listAcceptDay.add(WeekDay.SU);
			listAcceptDay.add(WeekDay.MO);
			listAcceptDay.add(WeekDay.TU);
			listAcceptDay.add(WeekDay.WE);
			listAcceptDay.add(WeekDay.TH);
			listAcceptDay.add(WeekDay.FR);
			listAcceptDay.add(WeekDay.SA);

			for (int i = 0; i < listAcceptDay.size() || i < c.length; i++) {
				char day = c[i];
				if (!"0".equals(String.valueOf(day))) {
					listDay.add(listAcceptDay.get(i));
				}
			}
		}
		return listDay;
	}

	public static int getDuration(DtStart startDate, DateProperty endDate) {
		if (startDate != null && endDate != null) {
			long start = startDate.getDate().getTime();
			long end = endDate.getDate().getTime();
			return (int) ((end - start) / 1000);
		}
		return 0;
	}

	public static int getPrivacy(VToDo vEvent) {
		if (vEvent.getClassification() != null
				&& Clazz.PRIVATE.equals(vEvent.getClassification())) {
			return 1;
		} else {
			return 0;
		}
	}

	public static void appendAlert(Event event, ComponentList cl,
			PropertyList propertyList) {
		if (cl.size() > 0) {
			VAlarm valarm = (VAlarm) cl.get(0);
			if (valarm != null) {
				Trigger trigger = valarm.getTrigger();
				Dur dur = trigger.getDuration();
				event.setAlert(dur.getMinutes());
				return;
			}
		}
		event.setAlert(0);
	}

	public static void appendRecurence(Event event, CalendarComponent component) {
		RRule rrule = (RRule) component.getProperties().getProperty(
				Property.RRULE);

		EventRecurrence er = new EventRecurrence();

		if (rrule != null) {
			Recur recur = rrule.getRecur();
			Map<WeekDay, Integer> days = new HashMap<WeekDay, Integer>();
			days.put(WeekDay.SU, 0);
			days.put(WeekDay.MO, 0);
			days.put(WeekDay.TU, 0);
			days.put(WeekDay.WE, 0);
			days.put(WeekDay.TH, 0);
			days.put(WeekDay.FR, 0);
			days.put(WeekDay.SA, 0);

			for (Object ob : recur.getDayList()) {
				days.put((WeekDay) ob, 1);
			}

			StringBuilder sb = new StringBuilder();

			sb.append(days.get(WeekDay.SU));
			sb.append(days.get(WeekDay.MO));
			sb.append(days.get(WeekDay.TU));
			sb.append(days.get(WeekDay.WE));
			sb.append(days.get(WeekDay.TH));
			sb.append(days.get(WeekDay.FR));
			sb.append(days.get(WeekDay.SA));

			er.setDays(sb.toString());
			er.setEnd(recur.getUntil());
			PropertyList exdates = component.getProperties(Property.EXDATE);
			Set<Date> dates = new HashSet<Date>();
			for (Object ob : exdates) {
				ExDate exdate = (ExDate) ob;
				DateList dl = exdate.getDates();
				for (Object date : dl) {
					dates.add((Date) date);
				}
			}

			for (Date d : dates) {
				er.addException(d);
			}

			er.setFrequence(recur.getInterval());
			if (isEmpty(er.getDays()) || "0000000".equals(er.getDays())) {
				if (Recur.DAILY.equals(recur.getFrequency())) {
					er.setKind(RecurrenceKind.daily);
				} else if (Recur.WEEKLY.equals(recur.getFrequency())) {
					er.setKind(RecurrenceKind.weekly);
				} else if (Recur.MONTHLY.equals(recur.getFrequency())) {
					WeekDayList wdl = recur.getDayList();

					if (wdl.size() > 0) {
						er.setKind(RecurrenceKind.monthlybyday);
						GregorianCalendar cal = new GregorianCalendar();
						cal.setTime(event.getDate());
						cal.set(GregorianCalendar.WEEK_OF_MONTH, ((WeekDay) wdl
								.get(0)).getOffset());
						cal.set(GregorianCalendar.DAY_OF_WEEK, WeekDay
								.getCalendarDay((WeekDay) wdl.get(0)));
						event.setDate(cal.getTime());
					} else {
						er.setKind(RecurrenceKind.monthlybydate);
					}

				} else if (Recur.YEARLY.equals(recur.getFrequency())) {
					er.setKind(RecurrenceKind.yearly);
				}

			} else {
				er.setKind(RecurrenceKind.weekly);
			}
		}
		if (er.getKind() == null) {
			er.setKind(RecurrenceKind.none);
		}
		if (er.getFrequence() < 1) {
			er.setFrequence(1);
		}
		event.setRecurrence(er);
	}

	public static void appendAttendees(Event event, Component comp,
			AccessToken token) {
		List<Attendee> attendees = new ArrayList<Attendee>();
		for (Property prop : getProperties(comp, Property.ATTENDEE)) {
			Attendee att = new Attendee();
			Parameter param = prop.getParameter(Parameter.CN);
			if (param != null) {
				att.setDisplayName(param.getValue());
			}

			int mailIndex = prop.getValue().toLowerCase().indexOf("mailto:");
			if (mailIndex != -1) {
				att.setEmail(prop.getValue().substring(
						mailIndex + "mailto:".length()));
			}

			param = prop.getParameter(Parameter.ROLE);
			if (param != null) {
				int index = param.getValue().indexOf("-");
				if (index != -1) {
					att.setRequired(ParticipationRole.valueOf(param.getValue()
							.substring(0, index).replace("-", "")));
				}
			}

			PartStat partStat = (PartStat) prop
					.getParameter(Parameter.PARTSTAT);
			if (partStat != null) {
				if (partStat.equals(PartStat.IN_PROCESS)) {
					att.setState(ParticipationState.INPROGRESS);
				} else {
					att.setState(ParticipationState.getValueOf(partStat
							.getValue()));
				}

			}

			attendees.add(att);
		}
		event.setAttendees(attendees);
	}

	public static Calendar initCalandar() {
		Calendar calendar = new Calendar();

		calendar.getProperties().add(
				new ProdId("-//Aliasource Groupe LINAGORA//OBM Calendar //FR"));
		calendar.getProperties().add(Version.VERSION_2_0);
		calendar.getProperties().add(CalScale.GREGORIAN);

		return calendar;
	}

	public static Set<ExDate> getExDate(Event event) {
		Set<ExDate> ret = new HashSet<ExDate>();
		if (event.getRecurrence() != null
				&& event.getRecurrence().getExceptions() != null) {

			for (Date d : event.getRecurrence().getExceptions()) {

				boolean find = false;
				for (Event excp : event.getRecurrence().getEventExceptions()) {
					if (excp.getDate().equals(d)) {
						find = true;
						break;
					}
				}
				if (!find) {
					try {
						DateTime da = new DateTime(d);
						ExDate extd = new ExDate();

						extd.setValue(da.toString());

						ret.add(extd);
					} catch (ParseException e) {
						logger.error(e.getMessage(), e);
					}
				}
			}
		}
		return ret;
	}

	public static VAlarm getVAlarm(Integer alert) {
		if (alert != null && !alert.equals(0)) {
			Dur dur = new Dur(0, 0, -alert, 0);
			VAlarm va = new VAlarm(dur);
			Trigger ti = va.getTrigger();
			ti.getParameters().add(new Value("DURATION"));
			return va;
		}
		return null;
	}

	public static Clazz getClazz(int privacy) {
		if (0 == privacy) {
			return Clazz.PUBLIC;
		} else {
			return Clazz.PRIVATE;
		}
	}

	public static Organizer getOrganizer(Event event) {
		Organizer orga = new Organizer();
		try {
			orga.getParameters().add(new Cn(event.getOwner()));
			if(event.getOwnerEmail() != null && !"".equals(event.getOwnerEmail())){
				orga.setValue("mailto:"+event.getOwnerEmail());
			}
		} catch (URISyntaxException e) {
			logger.error(e.getMessage(), e);
		}
		return orga;
	}

	public static Transp getTransp(EventOpacity eo) {
		Transp transp = Transp.OPAQUE;
		if (EventOpacity.OPAQUE.equals(eo)) {
			transp = Transp.OPAQUE;
		} else if (EventOpacity.TRANSPARENT.equals(eo)) {
			transp = Transp.TRANSPARENT;
		}
		return transp;
	}

	private static Due getDue(Date start, int duration) {
		if (start != null && duration >= 0) {
			int durationInMS = duration * 1000;
			DateTime dateTimeEnd = new DateTime(start.getTime() + durationInMS);
			return new Due(dateTimeEnd);
		}
		return null;
	}

	public static DtEnd getDtEnd(Date start, int duration, boolean isAllDays) {
		if (start != null && duration >= 0) {
			net.fortuna.ical4j.model.Date dateTimeEnd = null;
			if (isAllDays) {
				dateTimeEnd = new net.fortuna.ical4j.model.Date(
						start.getTime() + 43200000);
			} else {
				dateTimeEnd = new DateTime(start.getTime() + duration * 1000);
			}
			return new DtEnd(dateTimeEnd);
		}
		return null;
	}

	public static DtStart getDtStart(Event event) {
		net.fortuna.ical4j.model.Date dt = null;
		if (EventType.VEVENT.equals(event.getType()) && event.isAllday()) {
			dt = new net.fortuna.ical4j.model.Date(
					event.getDate().getTime() + 43200000);
		} else {
			dt = new DateTime(event.getDate());
		}
		return new DtStart(dt);
	}

	public static RecurrenceId getRecurrenceId(Event event) {
		DateTime dt = new DateTime(event.getDate());
		return new RecurrenceId(dt);
	}

	public static Role getRole(Attendee attendee) {
		Role role = Role.OPT_PARTICIPANT;
		if (ParticipationRole.CHAIR.equals(attendee.getRequired())) {
			role = Role.CHAIR;
		} else if (ParticipationRole.NON.equals(attendee.getRequired())) {
			role = Role.NON_PARTICIPANT;
		} else if (ParticipationRole.OPT.equals(attendee.getRequired())) {
			role = Role.OPT_PARTICIPANT;
		} else if (ParticipationRole.REQ.equals(attendee.getRequired())) {
			role = Role.REQ_PARTICIPANT;
		}

		return role;
	}

	public static Cn getCn(Attendee attendee) {
		if (isEmpty(attendee.getDisplayName())) {
			return new Cn(attendee.getEmail());
		} else {
			return new Cn(attendee.getDisplayName());
		}
	}

	public static PartStat getPartStat(Attendee attendee) {
		PartStat partStat = null;
		if (ParticipationState.ACCEPTED.equals(attendee.getState())) {
			partStat = PartStat.ACCEPTED;
		} else if (ParticipationState.COMPLETED.equals(attendee.getState())) {
			partStat = PartStat.COMPLETED;
		} else if (ParticipationState.DECLINED.equals(attendee.getState())) {
			partStat = PartStat.DECLINED;
		} else if (ParticipationState.DELEGATED.equals(attendee.getState())) {
			partStat = PartStat.DELEGATED;
		} else if (ParticipationState.INPROGRESS.equals(attendee.getState())) {
			partStat = PartStat.IN_PROCESS;
		} else if (ParticipationState.NEEDSACTION.equals(attendee.getState())) {
			partStat = PartStat.NEEDS_ACTION;
		} else if (ParticipationState.TENTATIVE.equals(attendee.getState())) {
			partStat = PartStat.TENTATIVE;
		}
		return partStat;
	}

	public static RRule getRRule(Event event) {
		RRule rrule = null;
		Recur recur = getRecur(event);
		if (recur != null) {
			rrule = new RRule(recur);
		}
		return rrule;
	}

	public static ComponentList getComponents(Calendar calendar,
			String component) {
		return calendar.getComponents(component);
	}

	@SuppressWarnings("unchecked")
	public static List<Property> getProperties(Component comp, String property) {
		List<Property> propsSet = new ArrayList<Property>();
		PropertyList propList = comp.getProperties(property);
		for (Iterator<Property> it = propList.iterator(); it.hasNext();) {
			Property prop = it.next();
			propsSet.add(prop);
		}
		return propsSet;
	}

	public static boolean isEmpty(String st) {
		return st == null || "".equals(st);
	}
	
}
