/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.utils;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.transaction.UserTransaction;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.base.ObmDbType;

import fr.aliacom.obm.services.constant.ConstantService;

/**
 * Helper functions for hibernate / datasource management
 * 
 * 
 */
public class ObmHelper {

	public static final String DATA_SOURCE = "java:comp/env/jdbc/ObmDS";

	private static DataSource ds;
	private static final Log logger = LogFactory.getLog(ObmHelper.class);;

	public static final String USER_TRANSACTION = "java:comp/UserTransaction";

	private static UserTransaction ut;

	private static ObmDbType type = ObmDbType.PGSQL;

	static {
		InitialContext context;
		try {
			context = new InitialContext();
			ut = (UserTransaction) context.lookup(USER_TRANSACTION);
			ds = (DataSource) context.lookup(DATA_SOURCE);
		} catch (NamingException e) {
			logger.error("Cannot locate datasource at " + "jdbc/ObmDS", e);
		}
		type = ObmDbType.valueOf(ConstantService.getInstance().getStringValue(
				"dbtype").trim());
	}

	/**
	 * Ferme tout le 'JDBC crap' avec les tests qui vont bien. Seuls les
	 * paramètres non nuls seront fermés.
	 * 
	 * @param con
	 * @param st
	 * @param rs
	 */
	public static void cleanup(Connection con, Statement st, ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
			}
		}
		if (st != null) {
			try {
				st.close();
			} catch (SQLException e) {
			}
		}
		if (con != null) {
			try {
				con.close();
			} catch (SQLException e) {
			}
		}
	}

	/**
	 * Permet d'obtenir une connection du pool. On obtient la même pendant toute
	 * la durée de la transaction.
	 * 
	 * @return Une connection du pool
	 * @throws SQLException
	 */
	public static Connection getConnection() throws SQLException {
		Connection con = ds.getConnection();
		if (getType() == ObmDbType.MYSQL) {
			Statement st = con.createStatement();
			st.execute("set time_zone='+00:00'");
			st.close();
		}
		return con;
	}

	/**
	 * Permet d'accéder à la transaction liée au thread courant.
	 * 
	 * @return la transaction courante
	 */
	public static UserTransaction getUserTransaction() {
		return ut;
	}

	public static void rollback(UserTransaction ut) {
		try {
			ut.rollback();
		} catch (Exception e) {
			logger.error("Error while rolling-back", e);
		}
	}

	public static int lastInsertId(Connection con) throws SQLException {
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			if (type == ObmDbType.PGSQL) {
				rs = st.executeQuery("SELECT lastval()");
			} else {
				rs = st.executeQuery("SELECT last_insert_id()");
			}
			rs.next();
			return rs.getInt(1);
		} finally {
			cleanup(null, st, rs);
		}
	}

	/**
	 * Allocate a new entity id in the obm 2.2 entity table
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	private static int allocateEntityId(Connection con) throws SQLException {
		Statement st = null;
		try {
			st = con.createStatement();
			st
					.executeUpdate("insert into Entity (entity_mailing) values (TRUE)");
		} finally {
			ObmHelper.cleanup(null, st, null);
		}
		return lastInsertId(con);
	}

	/**
	 * call as linkEntity(con, UserEntity, user_id, X, Y) to link an entity to a
	 * user
	 * 
	 * @param con
	 * @param linkTable
	 * @param targetField
	 * @param entityId
	 * @param targetId
	 * @return
	 * @throws SQLException
	 */
	public static LinkedEntity linkEntity(Connection con, String linkTable,
			String targetField, int targetId) throws SQLException {
		Statement st = null;

		Integer existing = fetchEntityId(con, linkTable.replace("Entity", ""),
				targetId);
		if (existing != null && existing > 0) {
			return new LinkedEntity(targetId, existing);
		}
		int eid = allocateEntityId(con);

		try {
			st = con.createStatement();
			st.executeUpdate("INSERT INTO " + linkTable + " ("
					+ linkTable.toLowerCase() + "_entity_id, "
					+ linkTable.toLowerCase() + "_" + targetField
					+ ") VALUES (" + eid + ", " + targetId + ")");
		} finally {
			ObmHelper.cleanup(null, st, null);
		}
		return new LinkedEntity(targetId, eid);
	}

	public static Date selectNow(Connection con) throws SQLException {
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery("SELECT now()");
			rs.next();
			return rs.getTimestamp(1);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	private static Integer fetchEntityId(Connection con, String tt, Integer uid)
			throws SQLException {
		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery("SELECT " + tt.toLowerCase()
					+ "entity_entity_id FROM " + tt + "Entity WHERE "
					+ tt.toLowerCase() + "entity_" + tt.toLowerCase() + "_id="
					+ uid);
			if (rs.next()) {
				return rs.getInt(1);
			} else {
				return null;
			}
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
	}

	public static Integer fetchEntityId(String tt, Integer uid)
			throws SQLException {
		Connection con = null;
		try {
			con = ObmHelper.getConnection();
			return fetchEntityId(con, tt, uid);
		} finally {
			ObmHelper.cleanup(con, null, null);
		}
	}

	public static ObmDbType getType() {
		return type;
	}
}