/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package org.obm.sync.server;

import java.util.List;

import org.obm.sync.auth.AccessToken;
import org.obm.sync.base.KeyList;
import org.obm.sync.book.BookItemsParser;
import org.obm.sync.book.BookType;
import org.obm.sync.book.Contact;
import org.obm.sync.items.ContactChanges;
import org.obm.sync.utils.DateHelper;

import fr.aliacom.obm.common.contact.AddressBookBindingImpl;

public class AddressBookHandler extends SecureSyncHandler {

	private AddressBookBindingImpl binding;
	private BookItemsParser bip;

	public AddressBookHandler() {
		binding = new AddressBookBindingImpl();
		bip = new BookItemsParser();
	}

	@Override
	public void handle(String method, ParametersSource params,
			XmlResponder responder) throws Exception {
		if ("isReadOnly".equals(method)) {
			isReadOnly(params, responder);
		} else if ("listBooks".equals(method)) {
			listBooks(params, responder);
		} else if ("getSync".equals(method)) {
			getSync(params, responder);
		} else if ("createContact".equals(method)) {
			createContact(params, responder);
		} else if ("createContactWithoutDuplicate".equals(method)) {
			createContactWithoutDuplicate(params, responder);
		} else if ("modifyContact".equals(method)) {
			modifyContact(params, responder);
		} else if ("removeContact".equals(method)) {
			removeContact(params, responder);
		} else if ("getContactFromId".equals(method)) {
			getContactFromId(params, responder);
		} else if ("getContactTwinKeys".equals(method)) {
			getContactTwinKeys(params, responder);
		} else if ("searchContact".equals(method)) {
			searchContact(params, responder);
		} else {
			responder.sendError("Cannot handle method '" + method + "'");
		}

	}

	private BookType type(ParametersSource params) {
		return BookType.valueOf(params.getParameter("book"));
	}

	private void getContactTwinKeys(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			KeyList ret = binding.getContactTwinKeys(at, type(params), bip
					.parseContact(params.getParameter("contact")));
			responder.sendKeyList(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getContactFromId(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Contact ret = binding.getContactFromId(at, type(params), p(params,
					"id"));
			if (ret != null) {
				responder.sendContact(ret);
			} else {
				responder.sendError("contact with id " + p(params, "id")
						+ " not found.");
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void removeContact(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Contact ret = binding.removeContact(at, type(params), p(params,
					"id"));
			responder.sendContact(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void modifyContact(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			String ct = p(params, "contact");
			Contact ret = binding.modifyContact(at, type(params), bip
					.parseContact(ct));
			responder.sendContact(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void createContactWithoutDuplicate(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Contact ret = binding.createContact(at, type(params), bip
					.parseContact(p(params, "contact")));
			responder.sendContact(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void createContact(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Contact ret = binding.createContact(at, type(params), bip
					.parseContact(p(params, "contact")));
			responder.sendContact(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getSync(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			ContactChanges cc = binding.getSync(at, type(params), DateHelper
					.asDate(p(params, "lastSync")));
			responder.sendContactChanges(cc);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void listBooks(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			BookType[] ret = binding.listBooks(at);
			String[] st = new String[ret.length];
			for (int i = 0; i < st.length; i++) {
				st[i] = ret[i].toString();
			}
			responder.sendArrayOfString(st);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void isReadOnly(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			boolean ret = binding.isReadOnly(at, type(params));
			responder.sendString("" + ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}
	
	private void searchContact(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		String query = p(params,"query");
		int limit = Integer.parseInt(p(params,"limit"));
		try {
			List<Contact> ret = binding.searchContact(at, query, limit);
			responder.sendListContact(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

}
