/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package org.obm.sync.server;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.obm.sync.auth.AccessToken;
import org.obm.sync.base.Category;
import org.obm.sync.base.KeyList;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.CalendarItemsParser;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.items.EventChanges;
import org.obm.sync.utils.DOMUtils;
import org.obm.sync.utils.DateHelper;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import fr.aliacom.obm.common.calendar.CalendarBindingImpl;

/**
 * Handles the following urls :
 * 
 * <code>/calendar/getSync?sid=xxx</code>
 * <code>/calendar/removeEvent?sid=xxx</code>
 * <code>/calendar/modifyEvent?sid=xxx</code>
 * <code>/calendar/createEvent?sid=xxx</code>
 * <code>/calendar/getEventFromId?sid=xxx</code>
 * <code>/calendar/listCalendars?sid=xxx</code>
 * <code>/calendar/getEventTwinKeys?sid=xxx</code>
 * <code>/calendar/getRefusedKeys?sid=xxx</code>
 * <code>/calendar/getUserEmail?sid=xxx</code>
 * <code>/calendar/getEventFromExtId?sid=xxx</code>
 * <code>/calendar/getListEventsFromIntervalDate?sid=xxx</code>
 * <code>/calendar/getAllEvents?sid=xxx</code>
 * <code>/calendar/getAllEventTimeUpdate?sid=xxx</code>
 * <code>/calendar/parseEvent?sid=xxx</code>
 * <code>/calendar/parseEvents?sid=xxx</code>
 * <code>/calendar/parseICS?sid=xxx</code>
 * @author tom
 * 
 */
public class EventHandler extends SecureSyncHandler {

	private CalendarBindingImpl binding;
	private CalendarItemsParser cip;

	public EventHandler(EventType type) {
		binding = new CalendarBindingImpl(type);
		cip = new CalendarItemsParser();
	}

	@Override
	public void handle(String method, ParametersSource params,
			XmlResponder responder) throws Exception {
		if (method.equals("getSync")) {
			getSync(params, responder);
		} else if (method.equals("removeEvent")) {
			removeEvent(params, responder);
		} else if (method.equals("modifyEvent")) {
			modifyEvent(params, responder);
		} else if (method.equals("createEvent")) {
			createEvent(params, responder);
		} else if (method.equals("getEventFromId")) {
			getEventFromId(params, responder);
		} else if (method.equals("listCalendars")) {
			listCalendars(params, responder);
		} else if (method.equals("listCategories")) {
			listCategories(params, responder);
		} else if (method.equals("getEventTwinKeys")) {
			getEventTwinKeys(params, responder);
		} else if (method.equals("getRefusedKeys")) {
			getRefusedKeys(params, responder);
		} else if (method.equals("getUserEmail")) {
			getUserEmail(params, responder);
		} else if (method.equals("getEventFromExtId")) {
			getEventFromExtId(params, responder);
		} else if (method.equals("getListEventsFromIntervalDate")) {
			getListEventsFromIntervalDate(params, responder);
		} else if (method.equals("getAllEvents")) {
			getAllEvents(params, responder);
		}else if (method.equals("getAllEventTimeUpdate")) {
			getAllEventTimeUpdate(params, responder);
		} else if (method.equals("parseEvent")) {
			parseEvent(params, responder);
		} else if (method.equals("parseEvents")) {
			parseEvents(params, responder);
		} else if (method.equals("parseICS")) {
			parseICS(params, responder);
		} else {
			logger.error("cannot handle method '" + method + "'");
		}

	}

	private void parseICS(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			String ics = params.getParameter("ics");
			List<Event> events = binding.parseICS(at,ics);
			responder.sendListEvent(events);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void parseEvents(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			List<Event> events = getEvents(params);
			String ics = binding.parseEvents(at,events);
			responder.sendString(ics);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void parseEvent(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Event event = getEvent(params);
			String ics = binding.parseEvent(at,event);
			responder.sendString(ics);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		} 
	}

	private void getAllEvents(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			List<Event> e = binding.getAllEvents(at, params.getParameter("calendar"), EventType.valueOf(params.getParameter("eventType"))); 
			responder.sendListEvent(e);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}
	
	private void getAllEventTimeUpdate(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			List<EventTimeUpdate> e = binding.getAllEventTimeUpdate(at, params.getParameter("calendar"), EventType.valueOf(params.getParameter("eventType"))); 
			responder.sendListEventTimeUpdate(e);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getListEventsFromIntervalDate(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			List<Event> e = binding.getListEventsFromIntervalDate(at, params
					.getParameter("calendar"), 
					DateHelper.asDate(params.getParameter("start")),
					DateHelper.asDate(params.getParameter("end")));

			responder.sendListEvent(e);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}

	}

	private void getEventFromExtId(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Event e = binding.getEventFromExtId(at, params
					.getParameter("calendar"), params.getParameter("extId"));
			if (e != null) {
				responder.sendEvent(e);
			} else {
				responder.sendError("not found");
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}

	}

	private void getUserEmail(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			String ue = binding.getUserEmail(at);
			responder.sendString(ue);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void listCategories(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			List<Category> ret = binding.listCategories(at);
			responder.sendCategories(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getRefusedKeys(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			KeyList ret = binding.getRefusedKeys(at, params
					.getParameter("calendar"), DateHelper.asDate(params
					.getParameter("since")));
			responder.sendKeyList(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getEventTwinKeys(ParametersSource params,
			XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			KeyList kl = binding.getEventTwinKeys(at, params
					.getParameter("calendar"), getEvent(params));
			responder.sendKeyList(kl);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void listCalendars(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			CalendarInfo[] lc = binding.listCalendars(at);
			responder.sendCalendarInformations(lc);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getEventFromId(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Event e = binding.getEventFromId(at, params
					.getParameter("calendar"), params.getParameter("id"));
			if (e != null) {
				responder.sendEvent(e);
			} else {
				responder.sendError("not found");
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void createEvent(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			String ev = binding.createEvent(at,
					params.getParameter("calendar"), getEvent(params));
			responder.sendString(ev);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private Event getEvent(ParametersSource params) throws SAXException,
			IOException, ParserConfigurationException,
			FactoryConfigurationError {
		String evAsString = params.getParameter("event");
		Document doc = DOMUtils.parse(new ByteArrayInputStream(evAsString
				.getBytes()));
		Event toModify = cip.parseEvent(doc.getDocumentElement());
		return toModify;
	}
	
	private List<Event> getEvents(ParametersSource params) throws SAXException, IOException, ParserConfigurationException, FactoryConfigurationError {
		String evsAsString = params.getParameter("events");
		Document doc = DOMUtils.parse(new ByteArrayInputStream(evsAsString
				.getBytes()));
		List<Event> toModify = cip.parseListEvents(doc.getDocumentElement());
		return toModify;
	}

	private void modifyEvent(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Event ev = binding.modifyEvent(at, params.getParameter("calendar"),
					getEvent(params), "true".equals(params
							.getParameter("updateAttendees")));
			if (ev != null) {
				responder.sendEvent(ev);
			} else {
				responder.sendError("Event did not exist.");
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void removeEvent(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Event ev = binding.removeEvent(at, params.getParameter("calendar"),
					params.getParameter("id"));
			if (ev != null) {
				responder.sendEvent(ev);
			} else {
				responder.sendError("Event did not exist.");
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

	private void getSync(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			EventChanges ret = binding.getSync(at, params
					.getParameter("calendar"), DateHelper.asDate(params
					.getParameter("lastSync")));
			responder.sendCalendarChanges(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}
	}

}
