package org.obm.sync.server;

import java.util.Map;

import org.obm.sync.auth.AccessToken;

import fr.aliacom.obm.common.setting.SettingBindingImpl;


/**
 * Handles the following urls :
 * 
 * <code>/setting/getSettings?sid=xxx</code>
 * 
 * @author adrien
 * 
 */
public class SettingHandler extends SecureSyncHandler{

	private SettingBindingImpl binding; 
	
	public SettingHandler(){
		this.binding = new SettingBindingImpl();
	}
	
	@Override
	public void handle(String method, ParametersSource params,
			XmlResponder responder) throws Exception {
		if ("getSettings".equals(method)) {
			getSettings(params, responder);
		} else {
			responder.sendError("Cannot handle method '" + method + "'");
		}
		
	}

	private void getSettings(ParametersSource params, XmlResponder responder) {
		AccessToken at = getToken(params);
		try {
			Map<String,String> ret = binding.getSettings(at);
			responder.sendSettings(ret);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			responder.sendError(e);
		}		
	}

}
