/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package org.obm.sync.server;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class SyncServlet extends HttpServlet {

	private static final long serialVersionUID = 3362408289739626998L;

	private Log logger = LogFactory.getLog(getClass());

	private Map<String, ISyncHandler> handlers;

	@Override
	protected void service(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		XmlResponder responder = new XmlResponder(resp);

		// /obm-sync/services
		String uri = req.getRequestURI();
		int idx = uri.indexOf("/services");

		if (idx < 0) {
			logger.warn("Invalid request uri: " + uri);
			responder.sendError(new Exception("Invalid request uri: " + uri));
			return;
		}

		idx += "/services".length();
		String query = uri.substring(idx);

		if (query.length() == 0) {
			showSyncStatus(req, resp);
		} else {
			try {
				handleQuery(query, req, responder);
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
				responder.sendError(e);
			}
		}

	}

	private void handleQuery(String query, HttpServletRequest req,
			XmlResponder responder) throws Exception {
		String[] q = query.split("/");
		if (q.length != 3) {
			logger.error("Query without 3 parts: " + query);
			responder
					.sendError(new Exception("Query without 3 parts: " + query));
		} else {
			String handlerKey = q[1];
			ISyncHandler handler = handlers.get(handlerKey);
			if (handler != null) {
				String method = q[2].substring(0, Math.max(q[2].length(), q[2]
						.lastIndexOf('?')));
				long t = System.nanoTime();
				handler.handle(method, new ParametersSource(req), responder);
				t = System.nanoTime() - t;
				logger.info("handler responded to " + handlerKey + "/" + method
						+ " in " + (t/1000000) + "ms.");
			} else {
				logger.error("no handler for " + q[1]);
				responder.sendError(new Exception("no handler for " + q[1]));
			}
		}

	}

	private void showSyncStatus(HttpServletRequest req, HttpServletResponse resp) {
		logger.info("show obm sync status");

		SyncStatus ss = new SyncStatus(handlers);
		ss.show(req, resp);
	}

	@Override
	public void init() throws ServletException {
		super.init();
		handlers = new HashMap<String, ISyncHandler>();
		handlers.put("login", new LoginHandler());
		handlers.put("calendar", new CalendarHandler());
		handlers.put("todo", new TodoHandler());
		handlers.put("book", new AddressBookHandler());
		handlers.put("setting", new SettingHandler());
	}

}
