/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package org.obm.sync.server;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.base.Category;
import org.obm.sync.base.KeyList;
import org.obm.sync.book.BookItemsWriter;
import org.obm.sync.book.Contact;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.CalendarItemsWriter;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.items.ContactChanges;
import org.obm.sync.items.EventChanges;
import org.obm.sync.setting.SettingItemsWriter;
import org.obm.sync.utils.DOMUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XmlResponder {

	private HttpServletResponse resp;
	private Log logger = LogFactory.getLog(getClass());
	private CalendarItemsWriter ciw;
	private BookItemsWriter biw;
	private SettingItemsWriter siw;

	public XmlResponder(HttpServletResponse resp) {
		this.resp = resp;
		this.ciw = new CalendarItemsWriter();
		this.biw = new BookItemsWriter();
		this.siw = new SettingItemsWriter();
	}

	public void sendError(String s) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/error.xsd", "error");
			Element root = doc.getDocumentElement();
			DOMUtils.createElementAndText(root, "message", s);
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendError(Exception e) {
		sendError(e.getMessage());
	}

	public void sendToken(AccessToken at) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/token.xsd", "token");
			Element root = doc.getDocumentElement();
			DOMUtils.createElementAndText(root, "sid", at.getSessionId());

			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	private void sendDom(Document doc) {
		try {
			resp.setContentType("text/xml;charset=UTF-8");
			DOMUtils.serialise(doc, resp.getOutputStream());
		} catch (Exception e) {
		}
	}

	public void sendString(String value) {
		sendArrayOfString(new String[] { value });
	}

	public void sendArrayOfString(String[] ret) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/string.xsd", "string");
			Element root = doc.getDocumentElement();
			for (String value : ret) {
				DOMUtils.createElementAndText(root, "value", value);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendKeyList(KeyList ret) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/keylist.xsd", "keylist");
			Element root = doc.getDocumentElement();
			for (String key : ret.getKeys()) {
				DOMUtils.createElementAndText(root, "key", key);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendEvent(Event ev) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/event.xsd", "event");
			Element root = doc.getDocumentElement();
			ciw.appendEvent(root, ev);
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendCalendarChanges(EventChanges cc) {
		sendDom(ciw.writeChanges(cc));
	}

	public void sendCalendarInformations(CalendarInfo[] lc) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/calendarinfos.xsd",
					"calendar-infos");
			Element root = doc.getDocumentElement();
			for (CalendarInfo ci : lc) {
				ciw.appendInfo(root, ci);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendContact(Contact contact) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/contact.xsd", "contact");
			Element root = doc.getDocumentElement();
			biw.appendContact(root, contact);
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendContactChanges(ContactChanges cc) {
		sendDom(biw.writeChanges(cc));
	}

	public void sendCategories(List<Category> ret) {
		// TODO Auto-generated method stub
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/categories.xsd", "categories");
			Element root = doc.getDocumentElement();
			for (Category c : ret) {
				ciw.appendCategory(root, c);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendListEvent(List<Event> evs) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/events.xsd", "events");
			Element root = doc.getDocumentElement();
			for(Event ev : evs){
				ciw.appendEvent(root, ev);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendListEventTimeUpdate(List<EventTimeUpdate> evs) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/events.xsd",
					"eventTimeUpdates");
			Element root = doc.getDocumentElement();
			for(EventTimeUpdate etu : evs){
				ciw.appendEventTimeUpdate(root, etu);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendListContact(List<Contact> ret) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/contact.xsd", "contacts");
			Element root = doc.getDocumentElement();
			for(Contact contact : ret){
				biw.appendContact(root, contact);
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public void sendSettings(Map<String, String> ret) {
		Document doc;
		try {
			doc = DOMUtils.createDoc("http://www.obm.org/xsd/sync/settings.xsd",
					"contacts");
			Element root = doc.getDocumentElement();
			for (Entry<String, String> entry : ret.entrySet()) {
				siw.appendSetting(root, entry.getKey(), entry.getValue());
			}
			sendDom(doc);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}

	}
}
