package org.obm.sync.client.calendar;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;
import java.util.TimeZone;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.fortuna.ical4j.data.ParserException;

import org.obm.sync.ObmSyncTestCase;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.base.Category;
import org.obm.sync.base.KeyList;
import org.obm.sync.book.Contact;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.CalendarItemsWriter;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventRecurrence;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.calendar.ParticipationRole;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.calendar.RecurrenceKind;
import org.obm.sync.items.EventChanges;
import org.obm.sync.locators.CalendarLocator;

public class CalendarClientTests extends ObmSyncTestCase {

	protected CalendarClient cal;
	protected AccessToken token;

	protected Event getTestEvent() {
		Event ev = new Event();

		Calendar cal = new GregorianCalendar();
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		ev.setDate(cal.getTime());

		ev.setExtId(UUID.randomUUID().toString());

		ev.setTitle("fake rdv " + System.currentTimeMillis());
		ev.setOwner(p("calendar"));
		ev.setDuration(3600);
		ev.setLocation("tlse");

		List<Attendee> la = new LinkedList<Attendee>();

		Attendee at = new Attendee();
		at.setDisplayName(p("login"));
		at.setEmail(p("login"));
		at.setState(ParticipationState.ACCEPTED);
		at.setRequired(ParticipationRole.CHAIR);
		la.add(at);

		at = new Attendee();
		at.setDisplayName("noIn TheDatabase");
		at.setEmail("notin@mydb.com");
		at.setState(ParticipationState.NEEDSACTION);
		at.setRequired(ParticipationRole.OPT);
		la.add(at);

		// at = new Attendee();
		// at.setDisplayName("pouic");
		// at.setEmail("pouic@zz.com");
		// at.setState(ParticipationState.INPROGRESS);
		// at.setRequired(ParticipationRole.OPT);
		// la.add(at);

		ev.setAttendees(la);
		ev.setAlert(60);

		EventRecurrence er = new EventRecurrence();
		er.setKind(RecurrenceKind.daily);
		er.setFrequence(1);
		er.addException(ev.getDate());
		cal.add(Calendar.MONTH, 1);
		er.addException(cal.getTime());
		er.setEnd(null);

		ev.setRecurrence(er);
		return ev;
	}

	public void testDoSyncPerf() {
		testCreateEvent();

		try {
			EventChanges changes = cal.getSync(token, p("calendar"), null);
			assertNotNull(changes);
			assertNotNull(changes.getLastSync());
			assertNotNull(changes.getRemoved());
			assertNotNull(changes.getUpdated());

			assertTrue(changes.getUpdated().length > 0);
		} catch (Exception e) {
			e.printStackTrace();
			fail("Exception on getSync");
		}

		System.out.println("Start getSync() speed test...");
		int count = 100;
		try {
			long time = System.currentTimeMillis();
			for (int i = 0; i < count; i++) {
				EventChanges changes = cal.getSync(token, p("calendar"), null);
				assertNotNull(changes);
			}
			time = System.currentTimeMillis() - time;
			System.out.println(count + " getSync() calls took " + time
					+ "ms. Performing at " + (count * 1000) / time + "/sec");
		} catch (Exception e) {
			e.printStackTrace();
			fail("speed test failed");
		}

	}

	public void testDoSync() {
		try {
			EventChanges changes = cal.getSync(token, p("calendar"), null);
			assertNotNull(changes);
			assertNotNull(changes.getLastSync());
			assertNotNull(changes.getRemoved());
			assertNotNull(changes.getUpdated());

			assertTrue(changes.getUpdated().length > 0);
		} catch (Exception e) {
			e.printStackTrace();
			fail("Exception on getSync");
		}

	}
	
	public void testLastSync() throws Exception {
		Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		
		
		EventChanges changes = cal.getSync(token, p("calendar"), null);
		c.setTimeInMillis(changes.getLastSync().getTime());
		System.err.println("lastSync: "+c.getTime());
		
		Thread.sleep(1500);
		
		String created = cal.createEvent(token, p("calendar"), getTestEvent());
		System.err.println("created event "+created);

		changes = cal.getSync(token, p("calendar"), c.getTime());
		Event[] up = changes.getUpdated();
		c.setTimeInMillis(changes.getLastSync().getTime());
		System.err.println("found updated: "+up.length+" newLast: "+c.getTime());
		for (Event e : up) {
			System.err.println("  * e: "+e.getTitle()+" upd: "+e.getTimeUpdate());
		}
		assertTrue(up.length == 1);
	}

	public void testListCalendars() {
		try {
			CalendarInfo[] lc = cal.listCalendars(token);
			assertNotNull(lc);
			assertTrue(lc.length > 0);
			for (CalendarInfo ci : lc) {
				System.out.println("ci: " + ci.getMail() + " uid: "
						+ ci.getUid() + " read: " + ci.isRead() + " write: "
						+ ci.isWrite());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail("cannot list calendars");
		}
	}

	public void testCreateEvent() {
		try {
			String ret = cal.createEvent(token, p("calendar"), getTestEvent());
			CalendarItemsWriter ciw = new CalendarItemsWriter();
			System.out.println(ciw.getEventString(getTestEvent()));

			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testFindEvent() {
		try {
			String ret = cal.createEvent(token, p("calendar"), getTestEvent());
			CalendarItemsWriter ciw = new CalendarItemsWriter();
			System.out.println(ciw.getEventString(getTestEvent()));

			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			Event found = cal.getEventFromId(token, p("calendar"), ret);
			assertNotNull(found);
			assertTrue(found.getAttendees().size() >= 1);
			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(found.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}

			System.out.println("found event by id with title "
					+ found.getTitle());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testFindEventException() {
		try {

			CalendarItemsWriter ciw = new CalendarItemsWriter();
			// Event 1
			Event event1 = getTestEvent();

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);

			event1.getRecurrence().addEventException(event2);

			String ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			Event found = cal.getEventFromId(token, p("calendar"), ev1);
			assertNotNull(found);
			assertNotNull(found.getRecurrence());
			assertEquals(1, found.getRecurrence().getEventExceptions().size());

			System.out.println("found event by id with title "
					+ found.getTitle());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testFindEventByExtId() {
		try {
			Event ev = getTestEvent();
			String ret = cal.createEvent(token, p("calendar"), ev);
			CalendarItemsWriter ciw = new CalendarItemsWriter();
			System.out.println(ciw.getEventString(ev));

			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			Event found = cal.getEventFromExtId(token, p("calendar"), ev
					.getExtId());
			assertNotNull(found);
			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(found.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}
			assertTrue(found.getAttendees().size() >= 1);
			System.out.println("found event by extId with title "
					+ found.getTitle());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testFindTodoByExtId() {
		try {
			Event ev = getTestEvent();
			for (Attendee att : ev.getAttendees()) {
				att.setPercent(20);
			}

			String ret = cal.createEvent(token, p("calendar"), ev);
			CalendarItemsWriter ciw = new CalendarItemsWriter();
			System.out.println(ciw.getEventString(ev));

			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			Event found = cal.getEventFromExtId(token, p("calendar"), ev
					.getExtId());
			assertNotNull(found);
			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(found.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}
			for (Attendee att : found.getAttendees()) {
				assertEquals(20, att.getPercent());
			}

			assertTrue(found.getAttendees().size() >= 1);
			System.out.println("found event by extId with title "
					+ found.getTitle());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testModifyEvent() {
		try {
			Event ev = getTestEvent();
			String ret = cal.createEvent(token, p("calendar"), ev);
			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			ev.setUid(ret);
			ev.setTitle(ev.getTitle() + " modified");
			for (Attendee at : ev.getAttendees()) {
				at.setState(ParticipationState.DELEGATED);
			}
			ev.addAttendee(getTestAttendee());
			String title = ev.getTitle();

			ev = cal.modifyEvent(token, p("calendar"), ev, true);
			System.out.println("modified event with new title '"
					+ ev.getTitle() + "'");
			assertEquals(title, ev.getTitle());
			assertTrue(ev.getAttendees().size() > 0);
			for (Attendee at : ev.getAttendees()) {
				// assertEquals(at.getState(), ParticipationState.DELEGATED);
				System.out.println("new at " + at.getEmail() + " state: "
						+ at.getState());
			}

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(ev.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail("error on modifyEvent");
		}
	}

	public void testModifyEventLoop() {
		try {
			Event ev = getTestEvent();
			String ret = cal.createEvent(token, p("calendar"), ev);
			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			ev.setUid(ret);
			ev.setTitle(ev.getTitle() + " modified");
			for (Attendee at : ev.getAttendees()) {
				at.setState(ParticipationState.DELEGATED);
			}
			ev.addAttendee(getTestAttendee());
			String title = ev.getTitle();

			int CNT = 1;

			for (int i = 0; i < CNT; i++) {
				ev = cal.modifyEvent(token, p("calendar"), ev, true);
				System.out.println("modified event with new title '"
						+ ev.getTitle() + "'");
				assertEquals(title, ev.getTitle());
				assertTrue(ev.getAttendees().size() > 0);
				Pattern p = Pattern
						.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
				Matcher m = p.matcher(ev.getOwnerEmail());
				if (!m.find()) {
					fail("EmailOwner isn't valid");
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail("error on modifyEvent");
		}
	}

	private Attendee getTestAttendee() {
		Attendee at = new Attendee();
		at.setDisplayName("The Added Attendee");
		at.setEmail("attendee@zz.com");
		at.setState(ParticipationState.DELEGATED);
		at.setRequired(ParticipationRole.OPT);
		return at;
	}

	public void testRemoveEvent() {
		try {
			String ret = cal.createEvent(token, p("calendar"), getTestEvent());
			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			System.out.println("Trying rm with id: " + ret);
			Event removed = cal.removeEvent(token, p("calendar"), ret);
			assertNotNull(removed);

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(removed.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}

			Event ev = cal.getEventFromId(token, p("calendar"), ret);
			assertNull(ev);
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testRemoveEventWithException() {
		try {
			CalendarItemsWriter ciw = new CalendarItemsWriter();

			// Event 1
			Event event1 = getTestEvent();
			String ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);
			event2.setParentId(Integer.parseInt(ev1));
			String ev2 = cal.createEvent(token, p("calendar"), event2);
			System.out.println(ciw.getEventString(event2));
			assertNotNull(ev2);

			System.out.println("Trying rm with id: " + ev1);
			Event removed = cal.removeEvent(token, p("calendar"), ev1);
			assertNotNull(removed);

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(removed.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}

			Event ev = cal.getEventFromId(token, p("calendar"), ev1);
			assertNull(ev);

			ev = cal.getEventFromId(token, p("calendar"), ev2);

			// FIXME disabled in 2.2 for mysql bug
			// assertNull(ev);
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testRemoveOnNonEvent() {
		Event removed = null;
		try {
			removed = cal.removeEvent(token, p("calendar"), "12345678");
			fail("client should send en error");
		} catch (Exception e) {
			assertNull(removed);
		}
	}

	public void testModifyOnNonEvent() {
		Event removed = getTestEvent();
		removed.setUid("12345678");
		try {
			removed = cal.modifyEvent(token, p("calendar"), removed, true);
			fail("client should send en error");
		} catch (Exception e) {
		}
	}

	public void testGetUserEmail() {
		try {
			String ret = cal.getUserEmail(token);
			assertNotNull(ret);
			System.out.println("Found email: " + ret);
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on getUserEmail");
		}
	}

	public void testGetRefusedKeys() {
		try {
			KeyList ret = cal.getRefusedKeys(token, p("calendar"), null);
			assertNotNull(ret);
			assertNotNull(ret.getKeys());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on getRefusedKeys");
		}
	}

	public void testGetEventTwinKeys() {
		try {
			Event ev = getTestEvent();
			KeyList ret = null;

			ev.setTitle("twin");
			System.out.println("twin date: " + ev.getDate());
			String uid = cal.createEvent(token, p("calendar"), ev);
			ret = cal.getEventTwinKeys(token, p("calendar"), ev);
			assertNotNull(ret);
			System.out.println("got keylist with " + ret.getKeys().size()
					+ " for " + ev.getTitle() + " at " + ev.getDate());
			assertTrue(ret.getKeys().size() > 0);

			cal.removeEvent(token, p("calendar"), uid);
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on getEventTwinKeys");
		}
	}

	public void testGetEventFromId() {
		try {
			Event origEv = getTestEvent();
			String ret = cal.createEvent(token, p("calendar"), origEv);
			assertNotNull(ret);
			System.out.println("Created event with id: " + ret);

			Event ev = cal.getEventFromId(token, p("calendar"), ret);
			assertNotNull(ev);
			System.out.println("getEventFromId found event '" + ev.getTitle()
					+ "'");
			assertEquals(origEv.getDate(), ev.getDate());

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			Matcher m = p.matcher(ev.getOwnerEmail());
			if (!m.find()) {
				fail("EmailOwner isn't valid");
			}

			System.out.println("Orig ev date: " + origEv.getDate()
					+ " found date: " + ev.getDate());
		} catch (Exception e) {
			e.printStackTrace();
			fail("error on getEventFromId");
		}
	}

	public void testListCategories() {
		try {
			List<Category> cats = cal.listCategories(token);
			assertNotNull(cats);
			System.out.println("categories size: " + cats.size());
			int count = 500;
			long time = System.currentTimeMillis();
			for (int i = 0; i < count; i++) {
				cal.listCategories(token);
			}
			time = System.currentTimeMillis() - time;
			System.out.println(count + " listCategories() calls took " + time
					+ "ms. Performing at " + (count * 1000) / time + "/sec");

		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		}
	}

	public void testFreeBusy() throws IOException {
		String url = p("obm.sync.url").replace("/services",
				"/freebusy/thomas@zz.com");
		try {
			InputStream in = new URL(url).openStream();
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String line = null;
			while ((line = br.readLine()) != null) {
				System.out.println(line);
			}
		} catch (MalformedURLException e) {
		}
	}

	public void testGetListEventsFromIntervalDate() {
		String ev1 = "";
		String ev2 = "";
		String ev3 = "";
		String ev4 = "";
		String ev5 = "";
		String ev6 = "";
		try {
			CalendarItemsWriter ciw = new CalendarItemsWriter();

			Calendar calendar = new GregorianCalendar();
			calendar.set(Calendar.YEAR, 1990);

			// Event 1
			Event event1 = getTestEvent();
			event1.setRecurrence(null);
			calendar.set(Calendar.MINUTE, calendar.get(Calendar.MINUTE) - 10);
			event1.setDate(calendar.getTime());
			ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);
			calendar.set(Calendar.MINUTE, calendar.get(Calendar.MINUTE) - 60);
			event2.setDate(calendar.getTime());
			ev2 = cal.createEvent(token, p("calendar"), event2);
			System.out.println(ciw.getEventString(event2));
			assertNotNull(ev2);

			// Event 3
			Event event3 = getTestEvent();
			event3.setRecurrence(null);
			calendar.set(Calendar.MINUTE, calendar.get(Calendar.MINUTE) + 30);
			event3.setDate(calendar.getTime());
			ev3 = cal.createEvent(token, p("calendar"), event3);
			System.out.println(ciw.getEventString(event3));
			assertNotNull(ev3);

			// Event 4 which should not be in the result
			Event event4 = getTestEvent();
			event4.setRecurrence(null);
			calendar.set(Calendar.YEAR, 1991);
			event4.setDate(calendar.getTime());
			ev4 = cal.createEvent(token, p("calendar"), event4);
			System.out.println(ciw.getEventString(event4));
			assertNotNull(ev4);

			// Event 5 rec
			Event event5 = getTestEvent();
			calendar.set(Calendar.YEAR, 1989);
			event5.setDate(calendar.getTime());
			EventRecurrence er = new EventRecurrence();
			er.setKind(RecurrenceKind.monthlybydate);
			// er.setFrequence(1);
			calendar.set(Calendar.YEAR, 1991);
			er.setEnd(calendar.getTime());
			event5.setRecurrence(er);
			ev5 = cal.createEvent(token, p("calendar"), event5);
			System.out.println(ciw.getEventString(event5));
			assertNotNull(ev5);

			// Event 6 rec
			Event event6 = getTestEvent();
			calendar.set(Calendar.MONTH, 1);
			calendar.set(Calendar.YEAR, 1989);
			event6.setDate(calendar.getTime());
			EventRecurrence er6 = new EventRecurrence();
			er6.setKind(RecurrenceKind.daily);
			er6.setFrequence(10);
			calendar.set(Calendar.YEAR, 1991);
			er6.setEnd(calendar.getTime());
			event6.setRecurrence(er6);
			ev6 = cal.createEvent(token, p("calendar"), event6);
			System.out.println(ciw.getEventString(event6));
			assertNotNull(ev6);

			calendar = new GregorianCalendar();
			calendar.set(Calendar.YEAR, 1990);
			calendar.set(Calendar.HOUR, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			Date start = calendar.getTime();

			calendar.set(Calendar.DAY_OF_MONTH, calendar
					.get(Calendar.DAY_OF_MONTH) + 1);
			Date end = calendar.getTime();

			List<Event> found = cal.getListEventsFromIntervalDate(token,
					p("calendar"), start, end);

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			for (Event event : found) {
				Matcher m = p.matcher(event.getOwnerEmail());
				if (!m.find()) {
					fail("EmailOwner isn't valid");
				}
			}

			assertNotNull("result list must not be null", found);
			assertEquals("result list must contain only 5 events", 4, found
					.size());

			for (Event foundE : found) {
				assertFalse("event4 must not be in the list", ev4.equals(foundE
						.getUid()));
				assertFalse("event6 must not be in the list", ev6.equals(foundE
						.getUid()));
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail("error on createEvent");
		} finally {
			try {
				cal.removeEvent(token, p("calendar"), ev1);
				cal.removeEvent(token, p("calendar"), ev2);
				cal.removeEvent(token, p("calendar"), ev3);
				cal.removeEvent(token, p("calendar"), ev4);
				cal.removeEvent(token, p("calendar"), ev5);
				cal.removeEvent(token, p("calendar"), ev6);
			} catch (Exception e) {
			}
		}

	}

	public void testGetAllEvents() {
		try {
			CalendarItemsWriter ciw = new CalendarItemsWriter();

			// Event 1
			Event event1 = getTestEvent();
			event1.setRecurrence(null);
			String ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);
			Integer intEv1 = Integer.parseInt(ev1);
			event2.setParentId(intEv1.intValue());
			String ev2 = cal.createEvent(token, p("calendar"), event2);
			System.out.println(ciw.getEventString(event2));
			assertNotNull(ev2);

			List<Event> found = cal.getAllEvents(token, p("calendar"),
					EventType.VEVENT);
			assertNotNull(found);
			assertTrue(found.size() > 1);

			for (Event event : found) {
				if (ev2.equals(event.getUid())) {
					assertEquals(intEv1.intValue(), event.getParentId());
				}
			}

			Pattern p = Pattern
					.compile("[a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-z]{2,4}");
			for (Event event : found) {
				Matcher m = p.matcher(event.getOwnerEmail());
				if (!m.find()) {
					fail("EmailOwner isn't valid");
				}
			}
		} catch (Exception e) {
			fail("error on getAllEvents");
		}

	}

	public void testGetAllEventTimeUpdate() {
		try {
			CalendarItemsWriter ciw = new CalendarItemsWriter();

			// Event 1
			Event event1 = getTestEvent();
			event1.setRecurrence(null);
			String ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);
			Integer intEv1 = Integer.parseInt(ev1);
			event2.setParentId(intEv1.intValue());
			String ev2 = cal.createEvent(token, p("calendar"), event2);
			System.out.println(ciw.getEventString(event2));
			assertNotNull(ev2);

			List<EventTimeUpdate> found = cal.getAllEventTimeUpdate(token,
					p("calendar"), EventType.VEVENT);
			assertNotNull(found);
			assertTrue(found.size() >= 2);

		} catch (Exception e) {
			fail("error on getAllEvents");
		}

	}

	public void testParseEvents() {
		try {
			CalendarItemsWriter ciw = new CalendarItemsWriter();
			// Event 1
			Event event1 = getTestEvent();
			event1.setRecurrence(null);
			String ev1 = cal.createEvent(token, p("calendar"), event1);
			System.out.println(ciw.getEventString(event1));
			assertNotNull(ev1);

			// Event 2
			Event event2 = getTestEvent();
			event2.setRecurrence(null);
			String ev2 = cal.createEvent(token, p("calendar"), event2);
			System.out.println(ciw.getEventString(event2));
			assertNotNull(ev2);

			List<Event> le = new LinkedList<Event>();
			le.add(event1);
			le.add(event2);

			String ll = cal.parseEvents(token, le);

			assertNotNull(ll);
			assertFalse("".endsWith(ll));

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testParseEvent() {
		try {
			// Event 1
			Event event1 = getTestEvent();

			String ll = cal.parseEvent(token, event1);
			assertNotNull(ll);
			assertFalse("".endsWith(ll));

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testParseICS() throws IOException, ParserException {
		InputStream in = ObmSyncTestCase.class.getClassLoader()
				.getResourceAsStream("icsFile/eventComplet.ics");
		if (in == null) {
			fail("Cannot load event1.ics");
		}

		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		StringBuilder sb = new StringBuilder();
		String line = null;
		try {
			while ((line = reader.readLine()) != null) {
				sb.append(line + "\n");
			}
		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				fail(e.getMessage());
			}
		}

		String ics = sb.toString();
		List<Event> events;
		try {
			events = cal.parseICS(token, ics);
			assertEquals(1, events.size());
		} catch (Exception e) {
			fail(e.getMessage());
		}

	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		cal = new CalendarLocator().locate(p("obm.sync.url"));
		assertNotNull(cal);
		token = cal.login(p("login"), p("password"), "junit");
		assertNotNull(token);
	}

	@Override
	protected void tearDown() throws Exception {
		cal.logout(token);
		super.tearDown();
	}

	@Override
	protected Contact getTestContact() {
		return null;
	}

}
