package fr.aliasource.obm.tomcat;

import java.io.IOException;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Creates context paramets file suite for conf/Catalina/localhost directory
 * 
 * @author tom
 * 
 */
public class ContextConfigurationWriter {

	private Context ctx;

	public ContextConfigurationWriter(Context ctx) {
		this.ctx = ctx;
	}

	public void write(OutputStream o) throws IOException {
		try {
			Document doc = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder().newDocument();
			Element context = doc.createElement("Context");
			doc.appendChild(context);
			context.setAttribute("path", ctx.getContextPath());
			context.setAttribute("docBase", ctx.getDocBase());

			for (DbConnection c : ctx.getConnections()) {
				Element res = doc.createElement("Resource");
				context.appendChild(res);
				res.setAttribute("name", c.getJNDIName());
				res.setAttribute("auth", "Container");
				res.setAttribute("type", "javax.sql.DataSource");
				res.setAttribute("password", c.getPassword());
				res.setAttribute("url", c.getUrl());

				if (c.isAliapoolBased()) {
					res.setAttribute("factory",
							"fr.aliacom.jndi.DataSourceFactory");
					res.setAttribute("max", "4");
					res.setAttribute("pingQuery", "SELECT 1");

					Element ut = doc.createElement("ResourceLink");
					context.appendChild(ut);
					ut.setAttribute("name", "UserTransaction");
					ut.setAttribute("global", "UTransaction");
					ut
							.setAttribute("type",
									"javax.transaction.UserTransaction");
					res.setAttribute("login", c.getLogin());
					res.setAttribute("className", c.getDriver());
				} else {
					res.setAttribute("username", c.getLogin());
					res.setAttribute("driverClassName", c.getDriver());
					res.setAttribute("testWhileIdle", "true");
					res.setAttribute("validationQuery", "SELECT 1");
				}

			}
			serialise(doc, o);
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	public static void serialise(Document doc, OutputStream out)
			throws TransformerException {
		TransformerFactory fac = TransformerFactory.newInstance();
		Transformer tf = fac.newTransformer();
		tf.setOutputProperty(OutputKeys.INDENT, "yes");
		Source input = new DOMSource(doc.getDocumentElement());
		Result output = new StreamResult(out);
		tf.transform(input, output);
	}

}
