package fr.aliasource.obm.tomcat;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

import fr.aliasource.obm.conf.ObmConfig;

public class WriterApp implements IApplication {

	private Log logger = LogFactory.getLog(Activator.getDefault().getClass());

	public Object start(IApplicationContext context) throws Exception {
		logger.info("Context Configuration Writer");

		String[] args = (String[]) context.getArguments().get(
				"application.args");

		if (args != null && args.length > 0 && args.length % 2 == 0) {
			WriterParams wp = new WriterParams(new FileInputStream(args[0]));
			Context ctx = new Context(
					wp.getProperty(WriterParams.CONTEXT_PATH), wp
							.getProperty(WriterParams.CONTEXT_ROOT));

			for (int i = 0; i < args.length / 2; i++) {
				WriterParams p = new WriterParams(new FileInputStream(
						args[2 * i]));
				String obmConfIniPath = args[2 * i + 1];
				writeContextXml(ctx, p, obmConfIniPath);
			}

			ContextConfigurationWriter ccw = new ContextConfigurationWriter(ctx);

			ccw.write(new FileOutputStream(wp
					.getProperty(WriterParams.CONTEXT_CONF_PATH)));
			logger.info("context xml written to "
					+ wp.getProperty(WriterParams.CONTEXT_CONF_PATH));
		} else {
			if (args != null && args.length > 0) {
				for (String arg : args) {
					logger.error("args was " + arg);
				}
			} else {
				logger.error("null or empty argument list");
			}
			logger.error("usage: ccw (<write.params> <obm_conf.ini>)*");
		}

		return IApplication.EXIT_OK;
	}

	private void writeContextXml(Context ctx, WriterParams p,
			String obmConfIniPath) throws IOException, FileNotFoundException {
		logger.info("Setting context configuration");
		ObmConfig configObm = new ObmConfig(new FileInputStream(obmConfIniPath));
		configObm.parse();
		String host = configObm.getProperty("host");
		if (host == null || host.trim().length() == 0) {
			host = "127.0.0.1";
		}
		String db = configObm.getProperty("db");
		String user = configObm.getProperty("user");
		String password = configObm.getProperty("password");
		DbConnection con = new DbConnection();

		String dbtype = configObm.getProperty("dbtype").toLowerCase();
		if (dbtype.equals("mysql")) {
			writeMySQLProperties(host, db, con);
		} else if (dbtype.equals("pgsql")) {
			writePostgreSQLProperties(host, db, con);
		}
		con.setLogin(user);
		con.setPassword(password);
		con.setAliapoolBased("true".equals(p
				.getProperty(WriterParams.CONTEXT_ALIAPOOL_BASED)));
		con.setJNDIName(p.getProperty(WriterParams.CONTEXT_DS_JNDINAME));
		ctx.addDbConnection(con);

	}

	private void writeMySQLProperties(String host, String db, DbConnection con) {
		String url = "jdbc:mysql://"
				+ host
				+ "/"
				+ db
				+ "?relaxAutocommit=true&zeroDateTimeBehavior=convertToNull"
				+ "&jdbcCompliantTruncation=false&interactiveClient=true&serverTimezone="
				+ "GMT"
				+ "&useGmtMillisForDatetime=true"
				+ "&useUnicode=true&characterEncoding=utf8&characterSetResults=utf8&connectionCollation=utf8_general_ci&dontTrackOpenResources=true";

		con.setDriver(DbConnection.MYSQL_DRIVER);
		con.setUrl(url);
	}

	private void writePostgreSQLProperties(String host, String db,
			DbConnection con) {
		String url = "jdbc:postgresql://" + host + "/" + db;

		con.setDriver(DbConnection.POSTGRESQL_DRIVER);
		con.setUrl(url);
	}

	public void stop() {
	}

}
